#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import pytz
from persistent import Persistent
from datetime import datetime

from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.contained import Contained
from zope.container.btree import BTreeContainer
from zope.interface import implements
from zope.publisher.interfaces import IRequest
from zope.publisher.browser import TestRequest
from zope.security import management
from zope.security.proxy import removeSecurityProxy

from schooltool.contact.interfaces import IContact
from schooltool.relationship import RelationshipProperty
from schooltool.relationship.uri import URIObject

import interfaces


URICreator = URIObject(
    'http://schooltool.org/ns/intervention/creator',
    'Creator', 'The creator/sender of the intervention relationship.')
URIResponsible = URIObject(
    'http://schooltool.org/ns/intervention/responsible',
    'Responsible', 'The persons responsible relationship.')
URIAtOneTime = URIObject(
    'http://schooltool.org/ns/intervention/atonetime',
    'AtOneTime', 'The at one time responsible relationship.')
URIMessage = URIObject(
    'http://schooltool.org/ns/intervention/message',
    'Message', 'The intervention message.')
URIGoal = URIObject(
    'http://schooltool.org/ns/intervention/goal',
    'Goal', 'The intervention goal.')
URIContact = URIObject(
    'http://schooltool.org/ns/intervention/contact',
    'Contact', 'The contact being linked.')


def getRequest():
    """Return the request object for the current request.
       In the case of unit testing, return a TestRequest instance."""

    i = management.getInteraction()
    for p in i.participations:
        if IRequest.providedBy(p):
            return p
    return TestRequest()


def contactName(contact):
    contact = removeSecurityProxy(contact)
    return '%s %s' % (contact.first_name, contact.last_name)


def contactsName(contacts):
    contacts = [removeSecurityProxy(c) for c in contacts]
    unsorted = [((c.last_name, c.first_name), contactName(c)) 
                 for c in contacts]
    return [name for sort, name in sorted(unsorted)]


def contactsEmail(contacts):
    return sorted([contact.email for contact in contacts if contact.email])


class InterventionRoot(BTreeContainer):
    """Container of InterventionSchoolYear objects."""

    implements(interfaces.IInterventionRoot)


class InterventionSchoolYear(BTreeContainer):
    """Container of InteventionStudent objects."""

    implements(interfaces.IInterventionSchoolYear)


class InterventionStudent(BTreeContainer):
    """Container of the student's intervention containers."""

    implements(interfaces.IInterventionStudent)

    def __init__(self):
        super(InterventionStudent, self).__init__()


class InterventionMessages(BTreeContainer):
    """Container of Tier1 InteventionMessage objects."""

    implements(interfaces.IInterventionMessages)


class InterventionMessage(Persistent, Contained):
    """Intervention message about a given student."""

    implements(interfaces.IInterventionMessage, IAttributeAnnotatable)

    subject = None
    created = None

    sender = RelationshipProperty(URICreator, URIMessage, URIContact)

    recipients = RelationshipProperty(URIResponsible, URIMessage,
                                      URIContact)

    def __init__(self, body, subject=None, status_change=False):
        self.subject = subject
        self.body = body
        self.status_change = status_change
        self.created = pytz.UTC.localize(datetime.utcnow())


class InterventionGoals(BTreeContainer):
    """Container of InterventionGoal objects."""

    implements(interfaces.IInterventionGoals)


class InterventionGoal(Persistent, Contained):
    """Intervention goal for a given student."""
    implements(interfaces.IInterventionGoal, IAttributeAnnotatable)

    created = None

    creator = RelationshipProperty(URICreator, URIGoal, URIContact)

    _persons_responsible = RelationshipProperty(URIResponsible, URIGoal,
                                               URIContact)

    at_one_time_responsible = RelationshipProperty(URIAtOneTime, URIGoal,
                                                   URIContact)

    def __init__(self, goal, timeline,
                 presenting_concerns=u'', strengths=u'', indicators=u'',
                 intervention=u'', goal_met=False,
                 follow_up_notes=u''):
        self.presenting_concerns = presenting_concerns
        self.goal = goal
        self.strengths = strengths
        self.indicators = indicators
        self.intervention = intervention
        self.timeline = timeline
        self.goal_met = goal_met
        self.follow_up_notes = follow_up_notes
        self.notified = False
        self.created = pytz.UTC.localize(datetime.utcnow())

    @property
    def persons_responsible(self):
        return self._persons_responsible

    @persons_responsible.setter
    def persons_responsible(self, value):
        contacts = [IContact(contact) for contact in value]
        for contact in self._persons_responsible:
            if contact not in contacts:
                self._persons_responsible.remove(contact)
        for contact in contacts:
            if contact not in self._persons_responsible:
                self._persons_responsible.add(contact)
            if contact not in self.at_one_time_responsible:
                self.at_one_time_responsible.add(contact)



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2013 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 9.

Sets cache for URIObjects.
"""

from zope.app.generations.utility import getRootFolder
from zope.component.hooks import getSite, setSite

from schooltool.app.interfaces import ISchoolToolApplication

from schooltool.intervention.intervention import URIAtOneTime
from schooltool.intervention.intervention import URIContact
from schooltool.intervention.intervention import URICreator
from schooltool.intervention.intervention import URIGoal
from schooltool.intervention.intervention import URIMessage
from schooltool.intervention.intervention import URIResponsible


def requireURICache(app):
    cache = app['schooltool.relationship.uri']
    standard_uris = [
        URIAtOneTime,
        URIContact,
        URICreator,
        URIGoal,
        URIMessage,
        URIResponsible,
        ]
    for uri in standard_uris:
        cache.cache(uri)


def evolve(context):
    root = getRootFolder(context)
    old_site = getSite()

    assert ISchoolToolApplication.providedBy(root)
    setSite(root)

    requireURICache(root)
    setSite(old_site)



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2008 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 7.

Move message and goal id lists to new contact relationsship properties.
"""

import pytz

from zope.app.generations.utility import findObjectsProviding
from zope.app.generations.utility import getRootFolder
from zope.component.hooks import getSite, setSite

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.contact.interfaces import IContact, IContactable


def getContacts(ids, persons):
    for id in ids:
        if len(id) > 2 and id[-2] == ':':
            index = int(id[-1])
            id = id[:-2]
        else:
            index = 0
        if id not in persons:
            continue
        person = persons[id]
        if index:
            contacts = [contact for contact in IContactable(person).contacts]
            if index <= len(contacts):
                yield contacts[index - 1]
        else:
            yield IContact(person)


def fixMessage(message, persons):
    created = message.created  # activate __dict__
    if 'sender' in message.__dict__:
        message.created = created    # activate persistence
        sender_id = message.__dict__['sender']
        del message.__dict__['sender']
        for contact in getContacts([sender_id], persons):
            if contact not in message.sender:
                message.sender.add(contact)

    if 'recipients' in message.__dict__:
        message.created = created    # activate persistence
        recipient_ids = message.__dict__['recipients']
        del message.__dict__['recipients']
        for contact in getContacts(recipient_ids, persons):
            if contact not in message.recipients:
                message.recipients.add(contact)


def fixGoal(goal, persons):
    created = goal.created  # activate __dict__
    if 'creator' in goal.__dict__:
        goal.created = created    # activate persistence
        creator_id = goal.__dict__['creator']
        del goal.__dict__['creator']
        for contact in getContacts([creator_id], persons):
            if contact not in goal.creator:
                goal.creator.add(contact)

    if '_persons_responsible' in goal.__dict__:
        goal.created = created    # activate persistence
        responsible_ids = goal.__dict__['_persons_responsible']
        del goal.__dict__['_persons_responsible']
        for contact in getContacts(responsible_ids, persons):
            if contact not in goal._persons_responsible:
                goal._persons_responsible.add(contact)

    if 'at_one_time_responsible' in goal.__dict__:
        at_one_time_ids = goal.__dict__['at_one_time_responsible']
        del goal.__dict__['at_one_time_responsible']
        for contact in getContacts(at_one_time_ids, persons):
            goal.at_one_time_responsible.add(contact)


def evolve(context):
    root = getRootFolder(context)
    old_site = getSite()
    apps = findObjectsProviding(root, ISchoolToolApplication)

    for app in apps:
        setSite(app)
        interventionRoot = app.get(u'schooltool.interventions', {})
        persons = app.get('persons', {})
        for schoolYear in interventionRoot.values():
            for student in schoolYear.values():
                for message in student['messages'].values():
                    fixMessage(message, persons)
                for goal in student['goals'].values():
                    fixGoal(goal, persons)




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from zope.container.interfaces import IContainer, IContained
from zope.container.constraints import contains, containers
from zope.interface import Interface, implements
from zope.schema.interfaces import IList, IBool
from zope.schema import TextLine, Text, Bool, List, Date, Datetime

from schooltool.intervention import InterventionGettext as _


class IInterventionRootMarker(Interface):
    """A marker interface for the root intervention objects."""


class IInterventionStudentMarker(Interface):
    """A marker interface for the student intervention objects."""


class IInterventionMarker(Interface):
    """A marker interface for the actual intervention objects. 
       Presently, these include messages and goals."""


class IInterventionTabProxy(Interface):
    """An interface for getting the displayable value of a message or goal."""


class IStudentSchoolYearsProxy(Interface):
    """A marker interface for the student schoolyear proxy."""


class IStudentSchoolYearProxy(Interface):
    """A marker interface for the student schoolyear proxy."""


class IPersonListField(IList):
    """A field that represents a list of people."""


class PersonListField(List):
    """A field that represents a list of people."""

    implements(IPersonListField)


class IGoalMetField(IBool):
    """A field that represents the goal met boolean."""


class GoalMetField(Bool):
    """A field that represents the goal met boolean."""

    implements(IGoalMetField)


class IInterventionRoot(IContainer):
    """Container of IInterventionSchoolYear objects."""

    contains('schooltool.intervention.interfaces.IInterventionSchoolYear')


class IInterventionSchoolYear(IContainer):
    """Container of the IInteventionStudent objects."""

    contains('schooltool.intervention.interfaces.IInterventionStudent')
    containers(IInterventionRoot)


class IInterventionStudent(IContainer):
    """Container of the student's intervention containers."""

    contains(IContainer)
    containers(IInterventionSchoolYear)


class IInterventionMessages(IContainer):
    """Container of intervention messages."""

    contains('schooltool.intervention.interfaces.IInterventionMessage')
    containers(IInterventionStudent)


class IInterventionMessage(IContained):
    """Intervention message about a given student."""

    sender = List(
        title=u"List of contacts (relationship) for the sender(s)"
        )

    recipients = PersonListField(
        title=_("Recipients"),
        )

    subject = TextLine(
        title=_("Subject"),
        required=False,
        )

    body = Text(
        title=_("Message body")
        )

    status_change = Bool(
        title=u"Status Change Message",
        required=False
        )

    created = Datetime(
        title=u"Creation Date/Time",
        )

    containers(IInterventionMessages)


class IInterventionGoals(IContainer):
    """Container of the studnet's interventions goals."""

    contains('schooltool.intervention.interfaces.IInterventionGoal')
    containers(IInterventionStudent)


class IInterventionGoal(IContained):
    """intervention goal for a given student."""

    presenting_concerns = Text(
        title=_("Presenting concerns"),
        required=False,
        )

    goal = Text(
        title=_("Goal"),
        )

    strengths = Text(
        title=_("Strengths"),
        required=False,
        )

    indicators = Text(
        title=_("Indicators"),
        required=False,
        )

    intervention = Text(
        title=_("Intervention"),
        required=False,
        )

    timeline = Date(
        title=_("Timeline"),
        )

    persons_responsible = PersonListField(
        title=_("Persons responsible"),
        description=_(u"The selected people will receive messages regarding"
                        " this goal through their interventions inbox and"
                        " email (if selected below)."),
        )

    at_one_time_responsible = List(
        title=_("Persons who have been at one time responsible"),
        )

    goal_met = GoalMetField(
        title=u"Goal met",
        required=False,
        default=False,
        )

    follow_up_notes = Text(
        title=_("Follow up notes"),
        required=False
        )

    notified = Bool(
        title=u"Notification sent to persons responsible"
        )

    creator = List(
        title=u"List of contacts (relationship) for the creator(s)"
        )

    created = Datetime(
        title=u"Creation Date/Time",
        )

    containers(IInterventionGoals)


class IInterventionPersonsResponsible(Interface):
    """An interface for getting the persons_responsible of a message or goal."""


class IInterventionSchoolYearId(Interface):
    """An interface for getting the schoolYearId of a message or goal."""


class IInterventionType(Interface):
    """An interface for getting the intervention type as a string."""


class IInterventionGetter(Interface):
    """An interface for getting the displayable value of a message or goal."""


class ISectionMessagesProxy(Interface):
    """An interface for adapting the SectionMessagesProxy."""


class IStudentMessagesGoalsProxy(Interface):
    """An interface for adapting the StudentMessagesGoalsProxy."""


class IIndependentActivities(Interface):
    """Bridge interface to remove intervention dependency on gradebook."""




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Student Intervention browser views.
"""
import csv
from StringIO import StringIO
import tempfile
import urllib
import zipfile

from zope.app.form.browser.editview import EditView
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.catalog.interfaces import ICatalog
from zope.component import getUtility, getMultiAdapter, queryUtility
from zope.container.interfaces import INameChooser
from zope.intid.interfaces import IIntIds
from zope.publisher.browser import BrowserView
from zope.schema import Bool, Date
from zope.security.checker import canWrite
from zope.security.proxy import removeSecurityProxy
from zope.security import checkPermission
from zope.traversing.api import getParents
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.i18n import translate

from zc.table.column import GetterColumn

from z3c.form import field, button
import z3c.form.interfaces
from z3c.form import form as z3cform

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.browser import app
from schooltool.app.membership import URIGroup, URIMembership
from schooltool.intervention import InterventionGettext as _
from schooltool.common import format_message
from schooltool.common.inlinept import InheritTemplate
from schooltool.common.inlinept import InlineViewPageTemplate
from schooltool.contact.interfaces import IContact
from schooltool.course.interfaces import ISection, IInstructor
from schooltool.email.interfaces import IEmailUtility, IEmailContainer
from schooltool.email.mail import status_messages
from schooltool.intervention.catalog import getStudentFirstName, getStudentLastName
from schooltool.intervention import intervention, sendmail
from schooltool.intervention import interfaces
from schooltool.person.interfaces import IPerson
from schooltool.relationship import getRelatedObjects
from schooltool.report.browser.report import RequestReportDownloadDialog
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.schoolyear.interfaces import ISchoolYearContainer
from schooltool.securitypolicy.crowds import AdministrationCrowd
from schooltool.skin.skin import NavigationViewlet
from schooltool.skin import flourish
from schooltool.skin.flourish.viewlet import ViewletManager
from schooltool.table.catalog import IndexedTableFormatter, FilterWidget
from schooltool.table.catalog import IndexedGetterColumn
from schooltool.table.catalog import IndexedLocaleAwareGetterColumn
from schooltool.table.table import SchoolToolTableFormatter
from schooltool.table.interfaces import ITableFormatter
from schooltool.term.interfaces import ITerm, IDateManager


class InterventionNavigationViewlet(NavigationViewlet):
    """View class for rendering the Intervention tab."""

    def canRender(self):
        person = IPerson(self.request.principal, None)
        return person is not None

    def url(self):
        person = IPerson(self.request.principal)
        return absoluteURL(person, self.request) + '/intervention_tab'


class FlourishInterventionNavViewlet(flourish.page.LinkViewlet):

    @property
    def person(self):
        return IPerson(self.request.principal, None)

    @property
    def enabled(self):
        person = self.person
        if person is None or queryUtility(IDateManager).current_term is None:
            return False
        return True

    @property
    def url(self):
        person = self.person
        if person is None:
            return ''
        return absoluteURL(person, self.request) + '/intervention_tab'


class StudentTableFormatter(SchoolToolTableFormatter):
    """View class for rendering the student table."""

    def columns(self):
        first_name = GetterColumn(
            name='first_name',
            title=_(u'First Name'),
            getter=lambda i, f: i.first_name,
            cell_formatter=self.cell_formatter,
            subsort=True)
        last_name = GetterColumn(
            name='last_name',
            title=_(u'Last Name'),
            getter=lambda i, f: i.last_name,
            cell_formatter=self.cell_formatter,
            subsort=True)
        return [first_name, last_name]

    def items(self):
        if AdministrationCrowd(None).contains(self.request.principal):
            persons = ISchoolToolApplication(None)['persons']
            return list(persons.values())

        students = set()
        person = IPerson(self.request.principal, None)
        for section in IInstructor(person).sections():
            for student in section.members:
                students.add(student)
        for student in person.advisees:
            students.add(student)
        return list(students)

    def sortOn(self):
        return (("last_name", False),)

    def cell_formatter(self, value, item, formatter):
        app = ISchoolToolApplication(None)
        container = interfaces.IInterventionSchoolYear(app)
        url = absoluteURL(container, formatter.request)
        if item.username not in container:
            # need to auto-vivyfy because we give the user a link to it
            getMultiAdapter((item, ISchoolYear(container)),
                interfaces.IInterventionStudent)
        return '<a href="%s/%s">%s</a>' % (url, item.username, value)


class StudentFilterWidget(FilterWidget):
    """View class for rendering the student filter widget."""

    template = ViewPageTemplateFile('templates/student_filter.pt')
    parameters = ['SEARCH_STUDENT_NAME', 'WITH_GOALS_ONLY']

    def filter(self, items):
        username = IPerson(self.request.principal).username

        if 'CLEAR_STUDENT_SEARCH' in self.request:
            for parameter in self.parameters:
                self.request.form[parameter] = ''
            return items

        if 'SEARCH_STUDENT_NAME' in self.request:
            searchstr = self.request['SEARCH_STUDENT_NAME'].lower()
            new_items = []
            for item in items:
                targetstr = '%s %s' % (item.first_name.lower(),
                    item.last_name.lower())
                if searchstr in targetstr:
                    new_items.append(item)
                else:
                    targetstr = item.username.lower()
                    if searchstr in targetstr:
                        new_items.append(item)
            items = new_items

        if self.request.get('WITH_GOALS_ONLY'):
            app = ISchoolToolApplication(None)
            year = interfaces.IInterventionSchoolYear(app)
            new_items = []
            for item in items:
                username = item.username
                if username in year and len(year[username]['goals']):
                    new_items.append(item)
            items = new_items

        return items


class FlourishStudentFilterWidget(StudentFilterWidget):
    """Flourish view class for rendering the student filter widget."""

    template = ViewPageTemplateFile('templates/f_student_filter.pt')


class InterventionTableFormatter(IndexedTableFormatter):
    """View class for rendering the intervention table."""

    def columns(self):
        first_name = IndexedLocaleAwareGetterColumn(
            index='first_name',
            name='first_name',
            title=_(u'First Name'),
            getter=lambda i, f: getStudentFirstName(i),
            subsort=True)
        last_name = IndexedLocaleAwareGetterColumn(
            index='last_name',
            name='last_name',
            title=_(u'Last Name'),
            getter=lambda i, f: getStudentLastName(i),
            subsort=True)
        inter_type = GetterColumn(
            name='intervention_type',
            title='',
            getter=self.intervention_type_getter)
        inter = GetterColumn(
            name='intervention',
            title=_(u"Intervention"),
            cell_formatter=self.intervention_cell_formatter,
            getter=interfaces.IInterventionGetter)
        created = IndexedGetterColumn(
            index='created',
            name='created',
            cell_formatter=self.created_cell_formatter,
            title=_(u'Created'),
            getter=lambda i, f: i.created,
            subsort=True)
        return [first_name, last_name, inter_type, inter, created]

    def sortOn(self):
        return (("created", True),)

    def intervention_type_getter(self, item, formatter):
        if interfaces.IInterventionType(item) == 'message':
            return translate(_('Msg:'), context=formatter.request)
        else:
            return translate(_('Goal:'), context=formatter.request)

    def intervention_cell_formatter(self, value, item, formatter):
        url = absoluteURL(IPerson(item), formatter.request)
        year = interfaces.IInterventionSchoolYear(item)
        url += '/schoolyears/%s/%ss/%s' % (year.__name__,
                                           interfaces.IInterventionType(item),
                                           item.__name__)
        url += '?nexturl=%s' % self.request.getURL()
        return '<a href="%s">%s</a>' % (url, value)

    def created_cell_formatter(self, value, item, formatter):
        return '<span>%s</span>' % value.strftime('%x')


class InterventionFilterWidget(FilterWidget):
    """View class for rendering the intervention filter widget."""

    template = ViewPageTemplateFile('templates/filter.pt')
    parameters = ['SEARCH_NAME', 'GOALS_ONLY']

    def filter(self, items):
        catalog = ICatalog(self.context)
        user_contact = IContact(IPerson(self.request.principal))
        contact_intid = getUtility(IIntIds).getId(user_contact)
        pr_idx = catalog['persons_responsible']
        items = [item for item in items
                 if contact_intid in pr_idx.documents_to_values[item['id']]]

        if 'CLEAR_SEARCH' in self.request:
            for parameter in self.parameters:
                self.request.form[parameter] = ''
            return items

        if 'SEARCH_NAME' in self.request:
            fn_idx = catalog['first_name']
            ln_idx = catalog['last_name']
            searchstr = self.request['SEARCH_NAME'].lower()
            new_items = []
            for item in items:
                targetstr = '%s %s' % (
                    fn_idx.documents_to_values[item['id']].lower(),
                    ln_idx.documents_to_values[item['id']].lower())
                if searchstr in targetstr:
                    new_items.append(item)
            items = new_items

        if self.request.get('GOALS_ONLY'):
            type_idx = catalog['intervention_type']
            items = [item for item in items
                     if type_idx.documents_to_values[item['id']] == 'goal']

        return items


class FlourishInterventionFilterWidget(InterventionFilterWidget):
    """Flourish view class for rendering the intervention filter widget."""

    template = ViewPageTemplateFile('templates/f_filter.pt')

    @property
    def show_goals(self):
        if 'form-submitted' in self.request:
            return self.request.get('SHOW_GOALS')
        else:
            return 'checked'

    @property
    def show_messages(self):
        if 'form-submitted' in self.request:
            return self.request.get('SHOW_MESSAGES')
        else:
            return 'checked'

    def filter(self, items):
        catalog = ICatalog(self.context)
        user_contact = IContact(IPerson(self.request.principal))
        contact_intid = getUtility(IIntIds).getId(user_contact)
        pr_idx = catalog['persons_responsible']
        items = [item for item in items
                 if contact_intid in pr_idx.documents_to_values[item['id']]]

        if self.request.get('SEARCH_NAME'):
            fn_idx = catalog['first_name']
            ln_idx = catalog['last_name']
            searchstr = self.request['SEARCH_NAME'].lower()
            new_items = []
            for item in items:
                targetstr = '%s %s' % (
                    fn_idx.documents_to_values[item['id']].lower(),
                    ln_idx.documents_to_values[item['id']].lower())
                if searchstr in targetstr:
                    new_items.append(item)
            items = new_items

        if self.show_messages and self.show_goals:
            return items
        elif self.show_goals:
            compare = 'goal'
        elif self.show_messages:
            compare = 'message'
        else:
            return []

        type_idx = catalog['intervention_type']
        items = [item for item in items
                 if type_idx.documents_to_values[item['id']] == compare]

        return items


class InterventionStartupView(BrowserView):
    """Dashboard view containing the user's intreventions."""

    title = _("Intervention")
    student_title = _("Search Students")
    inbox_title = _("Inbox")

    def __call__(self):
        if queryUtility(IDateManager).current_term is None:
            template = ViewPageTemplateFile('templates/no_current_term.pt')
            return template(self)
        else:
            return super(InterventionStartupView, self).__call__()

    def createInterventionTableFormatter(self, **kwargs):
        container = ISchoolToolApplication(None)['schooltool.interventions']
        formatter = getMultiAdapter((container, self.request),
                                    ITableFormatter)
        formatter.setUp(**kwargs)
        return formatter

    def setUpTables(self):
        self.available_table = self.createInterventionTableFormatter(
            prefix="intervention", batch_size=40)

    def update(self):
        self.setUpTables()


class FlourishInterventionStartupView(InterventionStartupView,
                                      flourish.page.Page):
    """Flourish Dashboard view containing the user's intreventions."""

    @property
    def no_data(self):
        return not self.available_table.filter(self.available_table.items())


class FlourishInterventionStartupLinks(flourish.page.RefineLinksViewlet):
    """flourish Intervention Dashboard search links viewlet."""


class InterventionStudentsView(BrowserView):
    """View to search for the user's student interventions."""

    title = _("Search Students")
    student_title = _("Students")

    def __call__(self):
        if queryUtility(IDateManager).current_term is None:
            template = ViewPageTemplateFile('templates/no_current_term.pt')
            return template(self)
        else:
            return super(InterventionStudentsView, self).__call__()

    def inboxURL(self):
        return absoluteURL(self.context, self.request)

    def createStudentTableFormatter(self, **kwargs):
        app = ISchoolToolApplication(None)
        container = interfaces.IInterventionSchoolYear(app)
        formatter = getMultiAdapter((container, self.request),
                                    ITableFormatter)
        formatter.setUp(**kwargs)
        return formatter

    def setUpTables(self):
        self.student_table = self.createStudentTableFormatter(
            prefix="student", batch_size=40)

    def update(self):
        self.setUpTables()


class FlourishInterventionStudentsView(InterventionStudentsView,
                                       flourish.page.Page):
    """Flourish view to search for the user's student intreventions."""


class InterventionStudentRedirectView(BrowserView):
    """View for redirecting to the no current term view."""

    def __call__(self):
        template = ViewPageTemplateFile('templates/no_current_term.pt')
        return template(self)


class InterventionStudentView(object):
    """View for managing a student's intervention."""

    @property
    def currentSchoolYear(self):
        return ISchoolYear(self.context.__parent__).title

    @property
    def isCurrentSchoolYear(self):
        app = ISchoolToolApplication(None)
        interventionSchoolYear = interfaces.IInterventionSchoolYear(app)
        return interventionSchoolYear == self.context.__parent__

    @property
    def schoolyears(self):
        results = []
        student = IPerson(self.context)
        app = ISchoolToolApplication(None)
        for year in ISchoolYearContainer(app).values():
            interventionSchoolYear = interfaces.IInterventionSchoolYear(year)
            interventionStudent = getMultiAdapter((student, year),
                interfaces.IInterventionStudent)

            result = {
                'url': absoluteURL(interventionStudent, self.request),
                'title': year.title,
                }
            results.append(result)
        return results

    def learnerOf(self):
        """Get the sections the student is a member of."""
        results = []

        for item in IPerson(self.context).groups:
            if ISection.providedBy(item):
                results.append(item)
            else:
                group_sections = getRelatedObjects(item, URIGroup,
                                                   rel_type=URIMembership)
                for section in group_sections:
                    results.append(section)
        results.sort(key=lambda s: s.__name__)
        return results

    def update(self):
        if 'schoolyear' in self.request:
            title = self.request['schoolyear']
            if title != self.currentSchoolYear:
                for year in self.schoolyears:
                    if year['title'] == title:
                        self.request.response.redirect(year['url'])

        interventionURL = absoluteURL(self.context, self.request)
        student = IPerson(self.context)

        self.studentURL = absoluteURL(student, self.request)
        self.studentName = '%s %s' % (IContact(student).first_name,
                                      IContact(student).last_name)

        self.messages = []
        for k, v in sorted(self.context['messages'].items()):
            if v.status_change:
                continue
            name = ', '.join(intervention.contactsName(v.sender))
            added = v.created
            message = {
                'creation_date': added,
                'url': absoluteURL(v, self.request),
                'text': _('Message from ${sender} sent ${date}',
                          mapping={'sender': name,
                                   'date': added.strftime('%x')})
                }
            self.messages.append(message)
        self.messages.sort(key=lambda m: m['creation_date'], reverse=True)
        self.newMessageURL = interventionURL + '/messages/+/addMessage.html'
        self.allMessagesURL = interventionURL + '/messages/allMessages.html'

        self.goals = []
        for k, v in sorted(self.context['goals'].items()):
            added = v.created.strftime('%x')
            due = v.timeline.strftime('%x')
            if v.goal_met:
                status = _('CLOSED')
            else:
                status = _('OPEN')
            goal = {
                'url': absoluteURL(v, self.request),
                'text': _('Goal ${goal_name} - Added ${date_added} - '
                          'Due ${date_due} - ${status}',
                          mapping={'goal_name': k,
                                   'date_added': added,
                                   'date_due': due,
                                   'status': status})
                }
            self.goals.append(goal)
        self.newGoalURL = interventionURL + '/goals/+/addGoal.html'
        self.allGoalsURL = interventionURL + '/goals/allGoals.html'

        studentId = IPerson(self.context).username
        currentYear = ISchoolYear(self.context.__parent__)
        terms = {}
        for section in self.learnerOf():
            if not canWrite(section, 'title'):
                continue
            if ISchoolYear(ITerm(section)) != currentYear:
                continue
            sheets = terms.setdefault(ITerm(section), [])
            activities = interfaces.IIndependentActivities(section, None)
            if activities is None:
                continue
            for worksheet in activities.values():
                if not worksheet.deployed:
                    continue
                course_title = ', '.join([course.title for course in
                                            section.courses])
                title = _('${worksheet_title} for ${course_title} - '
                          '${section_title}',
                          mapping={'worksheet_title': worksheet.title,
                                   'course_title': course_title,
                                   'section_title': section.title})
                worksheet_url = absoluteURL(worksheet, self.request)
                sheet = {
                    'url': '%s/gradebook/%s/view.html' % (worksheet_url,
                        studentId),
                    'text': title
                    }
                sheets.append(sheet)
        self.terms = [{'title': term.title, 'sheets': terms[term]}
                      for term in sorted(terms.keys())]

        self.statusMessages = []
        for k, v in sorted(self.context['messages'].items()):
            if not v.status_change:
                continue
            name = ', '.join(intervention.contactsName(v.sender))
            added = v.created.strftime('%x')
            message = {
                'url': absoluteURL(v, self.request),
                'text': _('Change of Status Message from ${sender} '
                          'sent ${date}',
                          mapping={'sender': name, 'date': added})
                }
            self.statusMessages.append(message)
        self.newStatusMessageURL = interventionURL + '/messages/+/addStatusMessage.html'
        self.allStatusMessagesURL = interventionURL + '/messages/allStatusMessages.html'


class EmailMixin(object):
    """Mixin class for getting messages for modal email sent notification."""

    @property
    def email(self):
        fail_message = _('The email could not be delivered to all recipients.')
        email_info = self.request.get('email')
        if email_info:
            messages = []
            email_container = IEmailContainer(ISchoolToolApplication(None))
            for info in email_info.split('|'):
                if '@' in info:
                    messages.append(translate(
                        format_message(
                            _('An email message was sent to: ${to_addresses}'),
                            mapping={'to_addresses': info}),
                        context=self.request))
                else:
                    messages.append(fail_message)
                    email = email_container.get(info, None)
                    if email is not None:
                        messages.append(translate(
                            format_message(
                                status_messages[email.status_code],
                                mapping=email.status_parameters),
                            context=self.request))
            return messages
        return ''


class FlourishInterventionStudentView(EmailMixin, flourish.page.Page):
    """Flourish View for managing a student's intervention."""

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    @property
    def subtitle(self):
        currentYear = ISchoolYear(self.context.__parent__)
        title = _(u'Interventions for ${year}',
                  mapping={'year': currentYear.title})
        return translate(title, context=self.request)

    @property
    def messages(self):
        messages = []
        for k, v in sorted(self.context['messages'].items()):
            messages.append({
                'creation_date': v.created,
                'url': absoluteURL(v, self.request),
                'from': ', '.join(intervention.contactsName(v.sender)),
                'subject': v.subject,
                'sent': v.created.strftime('%x'),
                })
        return sorted(messages, key=lambda m: m['creation_date'], reverse=True)

    @property
    def goals(self):
        goals = []
        for k, v in sorted(self.context['goals'].items()):
            name = v.goal
            if len(name) > 40:
                name = name[:37] + '...'
            goals.append({
                'url': absoluteURL(v, self.request),
                'name': name,
                'added': v.created.strftime('%x'),
                'due': v.timeline.strftime('%x'),
                'status': v.goal_met and _('CLOSED') or _('OPEN'),
                })
        return goals

    @property
    def canModify(self):
        return checkPermission('schooltool.edit', self.context)

    def learnerOf(self):
        """Get the sections the student is a member of."""
        results = []

        for item in IPerson(self.context).groups:
            if ISection.providedBy(item):
                results.append(item)
            else:
                group_sections = getRelatedObjects(item, URIGroup,
                                                   rel_type=URIMembership)
                for section in group_sections:
                    results.append(section)
        results.sort(key=lambda s: s.__name__)
        return results

    @property
    def sheets(self):
        studentId = IPerson(self.context).username
        currentYear = ISchoolYear(self.context.__parent__)
        sheets = []
        for section in self.learnerOf():
            if not canWrite(section, 'title'):
                continue
            term = ITerm(section)
            if ISchoolYear(term) != currentYear:
                continue
            activities = interfaces.IIndependentActivities(section, None)
            if activities is None:
                continue
            for worksheet in activities.values():
                if not worksheet.deployed:
                    continue
                course_title = ', '.join([course.title for course in
                                          section.courses])
                title = _('${worksheet_title} for ${course_title} - '
                          '${section_title}',
                          mapping={'worksheet_title': worksheet.title,
                                   'course_title': course_title,
                                   'section_title': section.title})
                worksheet_url = absoluteURL(worksheet, self.request)
                sheets.append({
                    'url': '%s/gradebook/%s/view.html' % (worksheet_url,
                        studentId),
                    'term': term.title,
                    'section': section.title,
                    'course': course_title,
                    'worksheet': worksheet.title,
                    })
        return sheets

    @property
    def schoolyears(self):
        results = []
        student = IPerson(self.context)
        app = ISchoolToolApplication(None)
        for year in ISchoolYearContainer(app).values():
            interventionSchoolYear = interfaces.IInterventionSchoolYear(year)
            interventionStudent = getMultiAdapter((student, year),
                interfaces.IInterventionStudent)

            result = {
                'url': absoluteURL(interventionStudent, self.request),
                'title': year.title,
                }
            results.append(result)
        return results

    def update(self):
        if 'schoolyear' in self.request:
            title = self.request['schoolyear']
            if title != ISchoolYear(self.context.__parent__).title:
                for year in self.schoolyears:
                    if year['title'] == title:
                        self.request.response.redirect(year['url'])


class FlourishInterventionStudentLinks(flourish.page.RefineLinksViewlet):
    """flourish Intervention Student add links viewlet."""


class InterventionStudentTertiaryNavigationManager(ViewletManager):

    template = InlineViewPageTemplate("""
        <ul tal:attributes="class view/list_class">
          <li tal:repeat="item view/items"
              tal:attributes="class item/class"
              tal:content="structure item/viewlet">
          </li>
        </ul>
    """)

    list_class = 'third-nav'

    @property
    def items(self):
        result = []
        schoolyears = ISchoolYearContainer(ISchoolToolApplication(None))
        active = ISchoolYear(removeSecurityProxy(self.context).__parent__)
        for schoolyear in schoolyears.values():
            url = '%s?schoolyear=%s' % (
                absoluteURL(self.context, self.request),
                schoolyear.__name__)
            result.append({
                'class': schoolyear.first == active.first and 'active' or None,
                'viewlet': u'<a href="%s">%s</a>' % (url, schoolyear.title),
                })
        return result


class FlourishInterventionStudentYearViewlet(flourish.viewlet.Viewlet,
                                             InterventionStudentView):
    template = InlineViewPageTemplate('''
    <form method="post"
          tal:attributes="action string:${context/@@absolute_url}">
      <select name="schoolyear" onchange="this.form.submit()">
        <tal:block repeat="year view/years">
          <option
              tal:attributes="value year/title;
                              selected year/selected"
              tal:content="year/title" />
        </tal:block>
      </select>
    </form>
    ''')

    @property
    def years(self):
        results = []
        student = IPerson(self.context)
        currentYear = removeSecurityProxy(
            interfaces.IInterventionSchoolYear(self.context))
        app = ISchoolToolApplication(None)
        for year in ISchoolYearContainer(app).values():
            interventionSchoolYear = interfaces.IInterventionSchoolYear(year)
            interventionStudent = getMultiAdapter((student, year),
                interfaces.IInterventionStudent)

            result = {
                'title': year.title,
                'selected': (interventionSchoolYear is currentYear and
                            'selected' or None),
                }
            results.append(result)
        return results

    def render(self, *args, **kw):
        return self.template(*args, **kw)


class FlourishRequestInterventionStudentReportView(RequestReportDownloadDialog):

    def nextURL(self):
        return absoluteURL(self.context, self.request) + '/intervention_student.pdf'


class SectionInterventionsView(object):
    """Tabbed section view showing links for adding or editing intervention data."""

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.heading = '%s - %s %s' % (', '.join([course.title for course in
                                               self.context.courses]),
                                       self.context.title,
                                       _("Interventions"))
        self.schoolyear = interfaces.IInterventionSchoolYear(
            ISchoolYear(self.context))
        activities = interfaces.IIndependentActivities(self.context, None)
        if activities is None:
            self.worksheets = []
        else:
            activities = removeSecurityProxy(activities)
            self.worksheets = [worksheet for worksheet in activities.values()
                               if worksheet.deployed]
        app = ISchoolToolApplication(None)
        self.activeSchoolYear = interfaces.IInterventionSchoolYear(app)

    def messageLinks(self, student):
        newMessageURL = '%s/messages/%s/+/addMessage.html' % (
            absoluteURL(self.context, self.request), student.__name__)
        allMessagesURL = '%s/messages/allMessages.html' % (
            absoluteURL(student, self.request))
        n_messages = len([m for m in student['messages'].values()
                          if not m.status_change])
        result = {
            'title': _('View Messages (${message_count})',
                       mapping={'message_count': n_messages}),
            'url': allMessagesURL,
            }
        results = [result]
        if self.schoolyear == self.activeSchoolYear:
            result = {
                'title': _('Write New'),
                'url': newMessageURL,
                }
            results.append(result)
        return (results)

    def goalLinks(self, student):
        student_url = absoluteURL(student, self.request)
        allGoalsURL = '%s/goals/allGoals.html' % student_url
        n_goals = len(student['goals'])
        return ({'title': _('View Goals (${goal_count})',
                            mapping={'goal_count': n_goals}),
                 'url': allGoalsURL},
                )

    def reportSheetLinks(self, student):
        studentId = student.__name__
        nexturl = absoluteURL(self.context, self.request) + \
            '/@@interventions.html'
        results = []
        for worksheet in self.worksheets:
            worksheet_url = absoluteURL(worksheet, self.request)
            result = {
                'title': worksheet.title,
                'url': '%s/gradebook/%s?nexturl=%s' % (worksheet_url,
                    studentId, nexturl)
                }
            results.append(result)
        return results

    @property
    def tabs(self):
        """Each tab is a tuple of tab_name, (tab_title, tab_link_factory)"""
        results = [
            ('messages', (_('Messages'), self.messageLinks)),
            ('goals', (_('Goals'), self.goalLinks)),
            ]
        if len(self.worksheets):
            result = ('sheets', (_('Report Sheets'), self.reportSheetLinks))
            results.append(result)
        return results


    def getActiveTabName(self):
        default_tab = 'messages'
        if len(self.worksheets):
            default_tab = 'sheets'
        return self.request.get('TAB', default_tab)

    def iterTabs(self):
        url = absoluteURL(self.context, self.request) + '/interventions.html'
        active_tab = self.getActiveTabName()
        for name, tab in self.tabs:
            title, link_factory = tab
            yield {
                'active': name == active_tab,
                'url': '%s?TAB=%s' % (url, name),
                'title': title
                }

    def listStudents(self):
        tab = dict(self.tabs).get(self.getActiveTabName())
        if tab is None:
            link_factory = lambda student: ()
        else:
            tab_title, link_factory = tab

        rows = []
        for student in self.context.members:
            interventionStudent = getMultiAdapter((student,
                ISchoolYear(self.context)), interfaces.IInterventionStudent)
            row = {
                'name': intervention.contactName(IContact(student)),
                'sortName': student.last_name + student.first_name,
                'url': absoluteURL(interventionStudent, self.request),
                'links': link_factory(interventionStudent),
                }
            rows.append(row)

        return sorted(rows, key=lambda row: row['sortName'])


class FlourishSectionInterventionsView(EmailMixin, flourish.page.Page,
                                       SectionInterventionsView):
    """Flourish tabbed section view showing links for adding or editing
       intervention data."""

    def __init__(self, context, request):
        SectionInterventionsView.__init__(self, context, request)

    @property
    def headings(self):
        headings = [_('Student')]
        if self.getActiveTabName() == 'messages':
            headings.append(_('Messages'))
            headings.append(_('Add'))
        elif self.getActiveTabName() == 'goals':
            headings.append(_('Goals'))
        else:
            headings.append(_('Report Sheets'))
        return headings

    def messageLinks(self, student):
        newMessageURL = '%s/messages/%s/addMessage.html' % (
            absoluteURL(self.context, self.request), student.__name__)
        allMessagesURL = '%s/messages/allMessages.html?nexturl=%s' % (
            absoluteURL(student, self.request), self.thisURL())
        n_messages = len([m for m in student['messages'].values()
                          if not m.status_change])
        results = [{
            'title': str(n_messages),
            'url': allMessagesURL,
            }]
        if self.schoolyear == self.activeSchoolYear:
            results.append({
                'title': '',
                'url': newMessageURL,
                })
        return results

    def goalLinks(self, student):
        student_url = absoluteURL(student, self.request)
        allGoalsURL = '%s/goals/allGoals.html?nexturl=%s' % (student_url,
            self.thisURL())
        n_goals = len(student['goals'])
        return [{
            'title': str(n_goals),
            'url': allGoalsURL,
            }]

    def reportSheetLinks(self, student):
        studentId = student.__name__
        results = []
        for worksheet in self.worksheets:
            worksheet_url = absoluteURL(worksheet, self.request)
            results.append({
                'title': worksheet.title,
                'url': '%s/gradebook/%s?nexturl=%s' % (worksheet_url,
                    studentId, self.thisURL())
                })
        return results

    def thisURL(self):
        url = absoluteURL(self.context, self.request)
        return url + '/interventions.html'


class SectionInterventionsTertiaryNavigationManager(
    flourish.page.TertiaryNavigationManager):

    template = InlineViewPageTemplate("""
        <ul tal:attributes="class view/list_class">
          <li tal:repeat="item view/items"
              tal:attributes="class item/class"
              tal:content="structure item/viewlet">
          </li>
        </ul>
    """)

    @property
    def items(self):
        result = []
        url = absoluteURL(self.context, self.request) + '/interventions.html'
        tabs = [
            ('messages', _('Messages')),
            ('goals', _('Goals')),
            ]
        activities = interfaces.IIndependentActivities(self.context, None)
        if activities is None:
            worksheets = []
        else:
            activities = removeSecurityProxy(activities)
            worksheets = [worksheet for worksheet in activities.values()
                          if worksheet.deployed]
        default_tab = 'messages'
        if len(worksheets):
            tabs.append(('sheets', _('Report Sheets')))
            default_tab = 'sheets'
        active_tab = self.request.get('TAB', default_tab)
        for tab, title in tabs:
            result.append({
                'class': tab == active_tab and 'active' or None,
                'viewlet': u'<a href="%s?TAB=%s">%s</a>' % (url, tab,
                           translate(title, context=self.request)),
                })
        return result


class InterventionMessageAddView(app.BaseAddView):
    """View for adding a message."""

    def __init__(self, context, request):
        super(InterventionMessageAddView, self).__init__(context, request)
        self.label = _("PLEASE REMEMBER: This information is part of the student's record.")

    def create(self, *args, **kw):
        self._sender = IContact(self.request.principal._person)
        self._recipients, body = args
        return self._factory(body)

    def add(self, message):
        """Add message to the student's messages."""
        context = self.context.context
        chooser = INameChooser(context)
        name = chooser.chooseName('', message)
        context[name] = message
        if self._sender:
            message.sender.add(self._sender)
        for recipient in self._recipients:
            message.recipients.add(IContact(recipient))
        sendmail.sendInterventionMessageEmail(message)
        return message

    def nextURL(self):
        """If section is in the context heirachcy, return to the section
           interventions view.  Otherwise return to the student's intervention
           center."""
        for parent in getParents(self.context):
            if ISection.providedBy(parent):
                return absoluteURL(parent, self.request) + '/interventions.html'
        interventionStudent = self.context.context.__parent__
        return absoluteURL(interventionStudent, self.request)


class IMessageAddForm(interfaces.IInterventionMessage):

    email = Bool(
        title=_(u"Email"),
        description=_(u"Send an email to the people selected above."),
        required=False,
        default=True)


class FlourishInterventionMessageAddView(flourish.form.AddForm):

    template = InheritTemplate(flourish.page.Page.template)
    label = None
    legend = _('Message Details')

    fields = field.Fields(IMessageAddForm)
    fields = fields.select('recipients', 'subject', 'body', 'email')

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    def updateWidgets(self):
        super(FlourishInterventionMessageAddView, self).updateWidgets()
        sender = IContact(self.request.principal._person)
        utility = getUtility(IEmailUtility)
        if not utility.enabled() or not sender.email:
            if False:
                self.widgets['email'].mode = z3c.form.interfaces.HIDDEN_MODE

    @button.buttonAndHandler(_('Submit'), name='add')
    def handleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            if data.get('email'):
                email = sendmail.sendInterventionMessageEmail(obj)
                if email.status_code is None:
                    info = ', '.join([urllib.quote(to)
                                      for to in email.to_addresses])
                else:
                    info = email.__name__
                url = '%s?email=%s' % (self.nextURL(), info)
                self.request.response.redirect(url)
            else:
                self._finishedAdd = True

    @button.buttonAndHandler(_("Cancel"))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.nextURL())

    def updateActions(self):
        super(FlourishInterventionMessageAddView, self).updateActions()
        self.actions['add'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')

    def create(self, data):
        message = intervention.InterventionMessage(data['body'],
                                                   data['subject'])
        self._sender = IContact(self.request.principal._person)
        self._recipients = data['recipients']
        return message

    def add(self, message):
        """Add message to the student's messages."""
        chooser = INameChooser(self.context)
        name = chooser.chooseName('', message)
        self.context[name] = message
        if self._sender:
            message.sender.add(IContact(self._sender))
        for recipient in self._recipients:
            message.recipients.add(IContact(recipient))
        return message

    def nextURL(self):
        """If section is in the context heirachcy, return to the section
           interventions view.  Otherwise return to the student's intervention
           center."""
        for parent in getParents(self.context):
            if ISection.providedBy(parent):
                return absoluteURL(parent, self.request) + '/interventions.html'
        interventionStudent = self.context.__parent__
        return absoluteURL(interventionStudent, self.request)


class InterventionStatusMessageAddView(app.BaseAddView):
    """View for adding a message."""

    def create(self, *args, **kw):
        self._sender = IContact(self.request.principal._person)
        self._recipients, body = args
        status_change = True
        return self._factory(body, status_change=status_change)

    def add(self, message):
        """Add message to the student's messages."""
        context = self.context.context
        chooser = INameChooser(context)
        name = chooser.chooseName('', message)
        context[name] = message
        if self._sender:
            message.sender.add(self._sender)
        for recipient in self._recipients:
            message.recipients.add(IContact(recipient))
        sendmail.sendInterventionMessageEmail(message)
        return message

    def nextURL(self):
        """If section is in the context heirachcy, return to the section
           interventions view.  Otherwise return to the student's intervention
           center."""
        for parent in getParents(self.context):
            if ISection.providedBy(parent):
                return absoluteURL(parent, self.request) + '/interventions.html'
        interventionStudent = self.context.context.__parent__
        return absoluteURL(interventionStudent, self.request)


class InterventionMessageView(object):
    """View for viewing a message."""

    def __init__(self, context, request):
        self.context = context
        self.request = request
        sender = ', '.join(intervention.contactsName(context.sender))
        added = context.created.strftime('%x')
        mapping = {'sender': sender, 'date': added}
        if context.status_change:
            self.heading = _('Change of Status Message from: ${sender} '
                             'sent ${date}', mapping=mapping)
        else:
            self.heading = _('Message from: ${sender} sent ${date}',
                             mapping=mapping)
        self.recipients = ', '.join(intervention.contactsName(
            context.recipients))


class FlourishInterventionMessageView(flourish.page.Page):
    """Flourish view for viewing a message."""

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.sender = ', '.join(intervention.contactsName(context.sender))
        self.added = context.created.strftime('%x')
        self.recipients = ', '.join(intervention.contactsName(
            context.recipients))

    @property
    def done(self):
        if 'nexturl' in self.request:
            return self.request['nexturl']
        return absoluteURL(self.context.__parent__.__parent__, self.request)


class InterventionMessagesView(object):
    """View for viewing all messages."""

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.messages = []
        for k, v in sorted(self.context.items()):
            if self.skipMessage(v):
                continue
            name = ', '.join(intervention.contactsName(v.sender))
            added = v.created
            heading = _('Message sent by ${sender} on ${date}',
                        mapping={'sender': name,
                                 'date': added.strftime('%x')})
            recipients = ', '.join(intervention.contactsName(v.recipients))
            message = {
                'creation_date': added,
                'heading': heading,
                'recipients': recipients,
                'subject': v.subject,
                'body': v.body,
            }
            self.messages.append(message)
        self.messages.sort(key=lambda m: m['creation_date'], reverse=True)

    def skipMessage(self, message):
        return message.status_change

    @property
    def heading(self):
        name = intervention.contactName(IContact(IPerson(self.context)))
        return _('Messages regarding ${student}', mapping={'student': name})


class FlourishInterventionMessagesView(flourish.page.Page):
    """Flourish view for viewing all messages."""

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    @property
    def subtitle(self):
        return _('Intervention Messages')

    @property
    def messages(self):
        messages = []
        for k, v in sorted(self.context.items()):
            self.sender = ', '.join(intervention.contactsName(v.sender))
            self.added = v.created.strftime('%x')
            self.recipients = ', '.join(intervention.contactsName(v.recipients))
            messages.append({
                'creation_date': v.created,
                'sender': ', '.join(intervention.contactsName(v.sender)),
                'added': v.created.strftime('%x'),
                'recipients': ', '.join(intervention.contactsName(
                                        v.recipients)),
                'subject': v.subject,
                'body': v.body,
            })
        return sorted(messages, key=lambda m: m['creation_date'], reverse=True)

    @property
    def done(self):
        if 'nexturl' in self.request:
            return self.request['nexturl']
        return absoluteURL(self.context.__parent__, self.request)


class InterventionStatusMessagesView(InterventionMessagesView):
    """View for viewing all change of status messages."""

    def skipMessage(self, message):
        return not message.status_change

    @property
    def heading(self):
        name = intervention.contactName(IContact(IPerson(self.context)))
        return _('Change of Status Messages regarding ${student}',
                 mapping={'student': name})


class MessagePersonLinks(flourish.page.RefineLinksViewlet):
    """Message Person links viewlet."""

    @property
    def title(self):
        student = removeSecurityProxy(IPerson(self.context))
        return "%s %s" % (student.first_name, student.last_name)


class MessagePersonInterventionsLink(flourish.page.LinkViewlet):

    @property
    def url(self):
        student = removeSecurityProxy(IPerson(self.context))
        return absoluteURL(student, self.request) + '/intervention'


class InterventionGoalAddView(app.BaseAddView):
    """View for adding a goal."""

    def create(self, *args, **kw):
        self._creator = IContact(self.request.principal._person)
        goal, timeline, self._persons_responsible = args
        return self._factory(goal, timeline,
                             kw['presenting_concerns'], kw['strengths'], kw['indicators'],
                             kw['intervention'], goal_met=kw['goal_met'],
                             follow_up_notes=kw['follow_up_notes'])

    def add(self, goal):
        """Add goal to the student's goals."""
        context = self.context.context
        chooser = INameChooser(context)
        name = chooser.chooseName('', goal)
        context[name] = goal
        goal.persons_responsible = self._persons_responsible
        if self._creator is not None:
            goal.creator.add(IContact(self._creator))
        sendmail.sendInterventionGoalAddEmail(goal)
        return goal

    def nextURL(self):
        interventionStudent = self.context.context.__parent__
        return absoluteURL(interventionStudent, self.request)


class IGoalAddForm(interfaces.IInterventionGoal):

    timeline = Date(
        title=_("Target date"),
        )

    email = Bool(
        title=_(u"Email"),
        description=_(u"Send an email to the people selected above."),
        required=False,
        default=True)


class SimpleFormAdapter(object):

    def __init__(self, context):
        self.__dict__['context'] = removeSecurityProxy(context)

    def __setattr__(self, name, value):
        setattr(self.context, name, value)

    def __getattr__(self, name):
        return getattr(self.context, name)


class FlourishInterventionGoalAddView(flourish.form.AddForm):

    template = InheritTemplate(flourish.page.Page.template)
    label = None
    legend = _('Goal Details')

    fields = field.Fields(IGoalAddForm)
    fields = fields.select('persons_responsible', 'presenting_concerns', 'goal',
                            'strengths', 'indicators', 'intervention',
                            'timeline', 'goal_met', 'follow_up_notes', 'email')

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    def updateWidgets(self):
        super(FlourishInterventionGoalAddView, self).updateWidgets()
        sender = IContact(self.request.principal._person)
        utility = getUtility(IEmailUtility)
        if not utility.enabled() or not sender.email:
            if False:
                self.widgets['email'].mode = z3c.form.interfaces.HIDDEN_MODE

    @button.buttonAndHandler(_('Submit'), name='add')
    def handleAdd(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        obj = self.createAndAdd(data)
        if obj is not None:
            if data.get('email'):
                info = []
                emails = sendmail.sendInterventionGoalAddEmail(obj)
                for email in emails:
                    if email.status_code is None:
                        info.append(', '.join([urllib.quote(to)
                                               for to in email.to_addresses]))
                    else:
                        info.append(email.__name__)
                url = '%s?email=%s' % (self.nextURL(), '|'.join(info))
                self.request.response.redirect(url)
            else:
                self._finishedAdd = True

    @button.buttonAndHandler(_("Cancel"))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.nextURL())

    def updateActions(self):
        super(FlourishInterventionGoalAddView, self).updateActions()
        self.actions['add'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')


    def create(self, data):
        creator = IContact(self.request.principal._person)
        goal = intervention.InterventionGoal(
            data['goal'], data['timeline'],
            data['presenting_concerns'], data['strengths'], data['indicators'],
            data['intervention'],
            goal_met=data['goal_met'], follow_up_notes=data['follow_up_notes'])
        self._persons_responsible = data['persons_responsible']
        self._creator = creator
        return goal

    def add(self, goal):
        """Add goal to the student's goals."""
        chooser = INameChooser(self.context)
        name = chooser.chooseName('', goal)
        self.context[name] = goal
        goal.persons_responsible = self._persons_responsible
        if self._creator is not None:
            goal.creator.add(IContact(self._creator))
        return goal

    def nextURL(self):
        student = interfaces.IInterventionStudent(IPerson(self.context))
        return  absoluteURL(student, self.request)


class InterventionGoalEditView(EditView):
    """View for editing a goal"""

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    def update(self):
        if 'CANCEL' in self.request:
            self.request.response.redirect(self.nextURL())
        else:
            status = EditView.update(self)
            if 'UPDATE_SUBMIT' in self.request and not self.errors:
                self.request.response.redirect(self.nextURL())
            return status

    def nextURL(self):
        return absoluteURL(self.context, self.request)


class FlourishInterventionGoalEditView(z3cform.EditForm, flourish.form.Form):

    template = InheritTemplate(flourish.page.Page.template)
    label = None
    legend = _('Goal Details')

    fields = field.Fields(IGoalAddForm)
    fields = fields.select('persons_responsible', 'presenting_concerns', 'goal',
                            'strengths', 'indicators', 'intervention',
                            'timeline', 'goal_met', 'follow_up_notes')

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    @button.buttonAndHandler(_('Submit'), name='apply')
    def handleApply(self, action):
        super(FlourishInterventionGoalEditView, self).handleApply.func(self, action)
        # XXX: hacky sucessful submit check
        if (self.status == self.successMessage or
            self.status == self.noChangesMessage):
            self.request.response.redirect(self.nextURL())

    @button.buttonAndHandler(_("Cancel"))
    def handle_cancel_action(self, action):
        self.request.response.redirect(self.nextURL())

    def updateActions(self):
        super(FlourishInterventionGoalEditView, self).updateActions()
        self.actions['apply'].addClass('button-ok')
        self.actions['cancel'].addClass('button-cancel')

    def nextURL(self):
        return absoluteURL(self.context, self.request)


class InterventionGoalView(object):
    """View for viewing a goal."""

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.timeline = context.timeline.strftime('%x')
        self.notified = ''
        if context.notified:
            self.notified = _('notification sent')
        self.persons_responsible = ', '.join(intervention.contactsName(
            context.persons_responsible))
        self.goal_met = _('No')
        if context.goal_met:
            self.goal_met = _('Yes')

    @property
    def heading(self):
        student = removeSecurityProxy(IPerson(self.context))
        name = intervention.contactName(IContact(student))
        added = self.context.created.strftime('%x')
        return _('Goal ${goal_name} for ${student} added ${date}',
                 mapping={'goal_name': self.context.__name__,
                          'student': name, 'date': added})


class FlourishInterventionGoalView(flourish.page.Page):
    """Flourish view for viewing a goal."""

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.creator = ', '.join(intervention.contactsName(context.creator))
        self.created = context.created.strftime('%x')
        self.timeline = context.timeline.strftime('%x')
        self.goal_met = context.goal_met and _('Yes') or _('No')
        self.persons_responsible = ', '.join(intervention.contactsName(
            context.persons_responsible))

    @property
    def canModify(self):
        return canWrite(self.context, 'goal')

    @property
    def done(self):
        if 'nexturl' in self.request:
            return self.request['nexturl']
        if interfaces.IStudentMessagesGoalsProxy.providedBy(
            self.context.__parent__):
            person = IPerson(self.request.principal, None)
            if person is None:
                return ''
            return absoluteURL(person, self.request) + '/intervention_tab'
        return absoluteURL(self.context.__parent__.__parent__, self.request)


class InterventionGoalsView(object):
    """View for viewing all goals."""

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.goals = []
        for k, v in sorted(self.context.items()):
            added = v.created.strftime('%x')
            heading = _('Goal ${goal_name} added ${date}',
                        mapping={'goal_name': k, 'date': added})
            notified = ''
            if v.notified:
                notified = _('notification sent')
            persons_responsible = intervention.contactsName(
                v.persons_responsible)
            goal_met = _('No')
            if v.goal_met:
                goal_met = _('Yes')
            goal = {
                'heading': heading,
                'presenting_concerns': v.presenting_concerns,
                'goal': v.goal,
                'strengths': v.strengths,
                'indicators': v.indicators,
                'intervention': v.intervention,
                'timeline': v.timeline.strftime('%x'),
                'notified': notified,
                'persons_responsible': persons_responsible,
                'goal_met': goal_met,
                'follow_up_notes': v.follow_up_notes,
            }
            self.goals.append(goal)

    @property
    def heading(self):
        return _('Goals for: ${first_name} ${last_name}',
            mapping={'first_name': IContact(IPerson(self.context)).first_name,
                     'last_name': IContact(IPerson(self.context)).last_name})


class FlourishInterventionGoalsView(flourish.page.Page):
    """Flourish view for viewing all goals."""

    @property
    def title(self):
        student = IContact(IPerson(self.context))
        return '%s %s' % (student.first_name, student.last_name)

    @property
    def goals(self):
        goals = []
        for k, v in sorted(self.context.items()):
            persons_responsible = ', '.join(intervention.contactsName(
                v.persons_responsible))
            goals.append({
                'obj': v,
                'timeline': v.timeline.strftime('%x'),
                'goal_met': v.goal_met and _('Yes') or _('No'),
                'persons_responsible': ', '.join(intervention.contactsName(
                    v.persons_responsible)),
                'canModify': canWrite(v, 'goal')
            })
        return goals

    @property
    def done(self):
        if 'nexturl' in self.request:
            return self.request['nexturl']
        return absoluteURL(self.context.__parent__, self.request)


class GoalPersonLinks(flourish.page.RefineLinksViewlet):
    """Goal Person links viewlet."""

    @property
    def title(self):
        student = removeSecurityProxy(IPerson(self.context))
        return "%s %s" % (student.first_name, student.last_name)


class GoalPersonInterventionsLink(flourish.page.LinkViewlet):

    @property
    def url(self):
        student = removeSecurityProxy(IPerson(self.context))
        return absoluteURL(student, self.request) + '/intervention'


class NotifyGoalsView(object):
    """View for notifying persons responsible for goals that
       have come due"""

    def __init__(self, context, request):
        self.context = context
        self.request = request
        goalsNotified = sendmail.sendInterventionGoalNotifyEmails()
        self.goals = []
        for notified in goalsNotified:
            name = intervention.contactName(IContact(IPerson(notified)))
            goal = {
                'text': _('${student} goal ${goal_name}',
                          mapping={'student': name,
                                   'goal_name': notified.__name__}),
                'url': absoluteURL(notified, request)
                }
            self.goals.append(goal)


class MessagesCSVView(BrowserView):

    def __call__(self):
        csvfile = StringIO()
        writer = csv.writer(csvfile, quoting=csv.QUOTE_ALL)
        row = ['year', 'student', 'date', 'sender', 'recipients', 'body']
        writer.writerow(row)
        root = interfaces.IInterventionRoot(self.context)
        for year in root.values():
            for student in year.values():
                for message in student['messages'].values():
                    created = message.created.strftime('%Y-%m-%d')
                    sender =  [IPerson(c).username for c in message.sender]
                    recipients = ', '.join(intervention.contactsEmail(
                        message.recipients))
                    body = message.body.replace('\n', '\\n')
                    body = body.replace('\r', '\\r')
                    row = [year.__name__, student.__name__, created,
                           ', '.join(sender), recipients, body]
                    row = [item.encode('utf-8') for item in row]
                    writer.writerow(row)
        return csvfile.getvalue().decode('utf-8')


class GoalsCSVView(BrowserView):

    def __call__(self):
        csvfile = StringIO()
        writer = csv.writer(csvfile, quoting=csv.QUOTE_ALL)
        row = ['year', 'student', 'created', 'creator', 'presenting_concerns',
               'goal', 'strengths', 'indicators', 'intervention', 'timeline',
               'persons_responsible', 'goal_met', 'follow_up_notes', 'notified']
        writer.writerow(row)
        root = interfaces.IInterventionRoot(self.context)
        for year in root.values():
            for student in year.values():
                for goal in student['goals'].values():
                    created = goal.created.strftime('%Y-%m-%d')
                    creator =  [IPerson(c).username for c in goal.creator]
                    timeline = goal.timeline.strftime('%Y-%m-%d')
                    goal_met = unicode(goal.goal_met)
                    notified = unicode(goal.notified)
                    persons_responsible = ', '.join(
                        intervention.contactsEmail(
                        goal.persons_responsible))
                    row = [year.__name__, student.__name__,
                           created, ', '.join(creator),
                           goal.presenting_concerns, goal.goal,
                           goal.strengths, goal.indicators, goal.intervention,
                           timeline, persons_responsible, goal_met,
                           goal.follow_up_notes, notified]
                    clean_row = []
                    for item in row:
                        if item is None:
                            item = u''
                        item = item.replace('\n', '\\n')
                        item = item.replace('\r', '\\r')
                        clean_row.append(item.encode('utf-8'))
                    writer.writerow(clean_row)
        return csvfile.getvalue().decode('utf-8')


class MessagesGoalsCSVView(BrowserView):

    def __call__(self):
        messages = MessagesCSVView(self.context, self.request)()
        goals = GoalsCSVView(self.context, self.request)()

        zfile = StringIO()
        archive = zipfile.ZipFile(zfile, "w")
        messages_file = tempfile.mktemp('.csv')
        open(messages_file, 'w').write(messages.encode('utf-8'))
        archive.write(messages_file, 'messages.csv', zipfile.ZIP_DEFLATED)
        goals_file = tempfile.mktemp('.csv')
        open(goals_file, 'w').write(goals.encode('utf-8'))
        archive.write(goals_file, 'goals.csv', zipfile.ZIP_DEFLATED)
        archive.close()

        data = zfile.getvalue()
        response = self.request.response
        response.setHeader('Content-Type', 'application/zip')
        response.setHeader('Content-Length', len(data))
        return data




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from datetime import date

import z3c.form.field
from zope.component import getUtility
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.i18n import translate

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.contact.interfaces import IContactable, IContact
from schooltool.email.interfaces import IEmailUtility
from schooltool.email.mail import Email
from schooltool.person.interfaces import IPerson

from schooltool.intervention import InterventionGettext as _
from interfaces import IInterventionGoal, IInterventionSchoolYear
import intervention


def sendInterventionMessageEmail(obj):
    """Send email corresponding to the intervention message just added."""

    # there can be only one sender - none may or may not be accepted by utility
    for contact in obj.sender:
        sender = contact.email
        sender_name = intervention.contactName(contact)
        break
    else:
        sender = ''
        sender_name = ''
    recipients = intervention.contactsEmail(obj.recipients)
    name = intervention.contactName(IContact(IPerson(obj)))
    mapping = {'student': name}
    if obj.subject:
        subject = obj.subject
    elif obj.status_change:
        subject = translate(_('INTERVENTION STATUS CHANGE: ${student}',
                              mapping=mapping),
                            context=intervention.getRequest())
    else:
        subject = translate(_('INTERVENTION MESSAGE: ${student}',
                              mapping=mapping),
                            context=intervention.getRequest())
    body = translate(_('${sender} writes:', mapping={'sender': sender_name}),
                     context=intervention.getRequest())
    body += '\n\n%s' % obj.body
    return sendEmail(sender, recipients, subject, body)


def sendInterventionGoalAddEmail(obj):
    """Send email corresponding to the intervention goal just added."""

    # there can be only one sender - none may or may not be accepted by utility
    for contact in obj.creator:
        sender = contact.email
        break
    else:
        sender = ''

    student = IPerson(obj)
    student_contacts = IContactable(student).contacts
    name = intervention.contactName(IContact(student))
    subject = translate(_('INTERVENTION GOAL ADDED: ${student}',
                          mapping={'student': name}),
                        context=intervention.getRequest())

    responsible, notified = [], []
    for contact in obj.persons_responsible:
        if IPerson(contact, None) is student or contact in student_contacts:
            notified.append(contact)
        else:
            responsible.append(contact)

    emails = []
    if responsible:
        recipients = intervention.contactsEmail(responsible)
        body = buildInterventionGoalAddBody(obj)
        emails.append(sendEmail(sender, recipients, subject, body))

    if notified:
        recipients = intervention.contactsEmail(notified)
        body = buildInterventionGoalAddBody(obj, False)
        emails.append(sendEmail(sender, recipients, subject, body))
    return emails


def buildInterventionGoalAddBody(obj, staff=True):
    name = intervention.contactName(IContact(IPerson(obj)))
    body = translate(_('The following goal was added for ${student}:',
                       mapping={'student': name}),
                     context=intervention.getRequest())
    body += '\n'
    fields = z3c.form.field.Fields(IInterventionGoal)
    fields = fields.omit('__name__', '__parent__', 'goal_met',
                         'follow_up_notes', 'notified', 'creator', 'created',
                         'at_one_time_responsible')
    for k, v in fields.items():
        text = obj.__getattribute__(k)
        if not text:
            continue
        body += '\n%s\n%s\n\n' % (v.field.title, '-' * len(v.field.title))
        if k == 'persons_responsible':
            names = [intervention.contactName(contact)
                     for contact in obj.persons_responsible]
            text = '\n'.join(sorted(names))
        elif k == 'timeline':
            text = obj.timeline.strftime('%x')
        body += '%s\n' % text
    if staff:
        url = absoluteURL(obj.__parent__.__parent__, intervention.getRequest())
        intervention_center = translate(_('Intervention Center'),
                                        context=intervention.getRequest())
        body += '\n%s\n%s\n' % (intervention_center, '-' * len(intervention_center))
        body += '\n%s\n' % url
    return body


def sendInterventionGoalNotifyEmails():
    """Loop through all goals for the current school year and send
       email to the persons responsible for the goal if:

       1) the goal's timeline is on or before today.
       2) an email for the goal has never been sent.
    """

    request = intervention.getRequest()
    now = date.today()
    goalsNotified = []
    app = ISchoolToolApplication(None)
    interventionSchoolYear = IInterventionSchoolYear(app)
    for interventionStudent in interventionSchoolYear.values():
        for interventionGoal in interventionStudent['goals'].values():
            if interventionGoal.timeline <= now and not interventionGoal.notified:
                notifyInterventionGoal(interventionGoal, request)
                goalsNotified.append(interventionGoal)
    return goalsNotified


def notifyInterventionGoal(interventionGoal, request):
    """Send email to the persons responsible for the given goal."""

    # there can be only one sender - none may or may not be accepted by utility
    for contact in interventionGoal.creator:
        sender = contact.email
        break
    else:
        sender = ''
 
    student = IPerson(interventionGoal)
    student_contacts = IContactable(student).contacts
    responsible = []
    for contact in interventionGoal.persons_responsible:
        if IPerson(contact, None) is student or contact in student_contacts:
            continue
        responsible.append(contact)
    recipients = intervention.contactsEmail(responsible)
 
    name = intervention.contactName(IContact(student))
    subject = translate(_('INTERVENTION GOAL DUE: ${student}',
                          mapping={'student': name}),
                        context=request)
    url = absoluteURL(interventionGoal, request) + '/@@editGoal.html'
    body = translate(_("Please follow the link below to update the follow "
                       "up notes and, if appropriate, the goal met status "
                       "of the intervention goal for ${student}.",
                       mapping={'student': name}),
                     context=request)
    body += '\n\n%s\n' % url

    sendEmail(sender, recipients, subject, body)
    interventionGoal.notified = True


def sendEmail(sender, recipients, subject, body):
    """send email through the schooltool email utility."""

    email = Email(sender, recipients, body, subject)
    utility = getUtility(IEmailUtility)
    utility.send(email)
    return email


class TestMailDelivery(object):
    """Email delivery utility used for testing"""

    def enabled(self):
        return True

    def send(self, email):
        print 'From:', email.from_address
        print 'To:', ', '.join(email.to_addresses)
        print 'Subject:', email.subject
        print email.body




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Unit tests for the intervention catalog.
"""
import unittest
import doctest
from datetime import datetime

from zope.app.testing import setup
from zope.component import provideAdapter, provideUtility
from zope.component.hooks import setSite
from zope.interface import implements
from zope.intid.interfaces import IIntIds
from zope.site.folder import Folder
from zope.site.site import LocalSiteManager

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.catalog import getAppCatalogs
from schooltool.person.interfaces import IPerson
from schooltool.contact.interfaces import IContact

from schooltool.intervention import interfaces, intervention
from schooltool.intervention.catalog import (getMessageType, getGoalType,
    getMessagePersonsResponsible, getGoalPersonsResponsible,
    updatePersonsResponsibleIndex)


class AppStub(Folder):
    implements(ISchoolToolApplication)


class StudentStub:
    implements(IPerson)
    def __init__(self, first_name, last_name):
        self.first_name = first_name
        self.last_name = last_name


class ContactStub:
    implements(IContact)
    def __init__(self, first_name, last_name):
        self.first_name = first_name
        self.last_name = last_name


class MessageStub:
    implements(interfaces.IInterventionMessage, interfaces.IInterventionMarker)
    created = datetime(2009, 9, 9)
    def __init__(self, recipients=[], id=0):
        self.recipients = recipients
        self.id = id


class GoalStub:
    implements(interfaces.IInterventionGoal, interfaces.IInterventionMarker)
    created = datetime(2010, 1, 1)
    def __init__(self, persons_responsible=[], id=0):
        self.persons_responsible = persons_responsible
        self.id = id


class SchoolYearStub:
    __name__ = '2009-2010'


class EventStub:
    def __init__(self, participant1, participant2):
        self.rel_type = intervention.URIResponsible
        self.participant1 = participant1
        self.participant2 = participant2


def doctest_catalogSetUp():
    """
    We'll need an IntIds stub where getId return the object itself for ease of
    testing.

        >>> class IntIdsStub(object):
        ...     def getId(self, obj):
        ...         return obj
        >>> provideUtility(IntIdsStub(), IIntIds)

    Ensure that we have catalogs

        >>> app = ISchoolToolApplication(None)
        >>> catalogs = getAppCatalogs(app)

    We'll create the catalog with InterventionCatalog.

        >>> from schooltool.intervention.catalog import InterventionCatalog
        >>> catalog_obj = InterventionCatalog(app)
        >>> catalog_obj()
        >>> catalog = InterventionCatalog.get()

    Tests for the 'first_name' index:

        >>> catalog['first_name'].index_doc(11, MessageStub())
        >>> catalog['first_name'].index_doc(22, GoalStub())
        >>> sorted(catalog['first_name'].documents_to_values.items())
        [(11, 'John'), (22, 'Jane')]

    Tests for the 'last_name' index:

        >>> catalog['last_name'].index_doc(11, MessageStub())
        >>> catalog['last_name'].index_doc(22, GoalStub())
        >>> sorted(catalog['last_name'].documents_to_values.items())
        [(11, 'Deere'), (22, 'Doe')]

    Tests for the 'persons_responsible' index:

        >>> message = MessageStub(['teacher2', 'advisor1'])
        >>> goal = GoalStub(['teacher1', 'advisor2'])
        >>> catalog['persons_responsible'].index_doc(11, message)
        >>> catalog['persons_responsible'].index_doc(22, goal)
        >>> sorted(catalog['persons_responsible'].documents_to_values.items())
        [(11, ['teacher2', 'advisor1']), (22, ['teacher1', 'advisor2'])]

    Tests for the 'created' index:

        >>> catalog['created'].index_doc(11, MessageStub())
        >>> catalog['created'].index_doc(22, GoalStub())
        >>> sorted(catalog['created'].documents_to_values.items())
        [(11, datetime.datetime(2009, 9, 9, 0, 0)),
         (22, datetime.datetime(2010, 1, 1, 0, 0))]

    Tests for the 'schoolYearId' index:

        >>> catalog['schoolYearId'].index_doc(11, MessageStub())
        >>> catalog['schoolYearId'].index_doc(22, GoalStub())
        >>> sorted(catalog['schoolYearId'].documents_to_values.items())
        [(11, '2009-2010'), (22, '2009-2010')]

    Tests for the 'intervention_type' index:

        >>> catalog['intervention_type'].index_doc(11, MessageStub())
        >>> catalog['intervention_type'].index_doc(22, GoalStub())
        >>> sorted(catalog['intervention_type'].documents_to_values.items())
        [(11, 'message'), (22, 'goal')]
    """


def doctest_updatePersonsResponsibleIndex():
    """
    We'll need an IntIds stub that returns the object itself if it's an int and
    the object's id attribute for messages and goals.  Also it always
    returns True for the 'in' operator.

        >>> class IntIdsStub(dict):
        ...     def getId(self, obj):
        ...         if type(obj) == type(0):
        ...             return obj
        ...         else:
        ...             return obj.id
        ...     def queryId(self, obj, default=None):
        ...         return self.getId(obj)
        >>> provideUtility(IntIdsStub(), IIntIds)

    Ensure that we have catalogs

        >>> app = ISchoolToolApplication(None)
        >>> catalogs = getAppCatalogs(app)

    We'll create the catalog with InterventionCatalog.

        >>> from schooltool.intervention.catalog import InterventionCatalog
        >>> catalog_obj = InterventionCatalog(app)
        >>> catalog_obj()
        >>> catalog = InterventionCatalog.get()

    First we'll create a message and goal and index them.  We set the
    persons_responsible to be a list of integers for the predictablilty
    of the test.

        >>> message = MessageStub([1, 3], 11)
        >>> goal = GoalStub([4, 2], 22)
        >>> catalog.index_doc(message.id, message)
        >>> catalog.index_doc(goal.id, goal)
        >>> sorted(catalog['persons_responsible'].documents_to_values.items())
        [(11, [1, 3]), (22, [4, 2])]

    We'll change the message to have a new value for recipients and
    create an event with the message as participant1.

        >>> message.recipients = [9, 7]
        >>> event = EventStub(message, None)

    When we call updatePersonsResponsibleIndex with the event, we see that
    that persons_responsible index entry has been updated for that message.

        >>> updatePersonsResponsibleIndex(event)
        >>> sorted(catalog['persons_responsible'].documents_to_values.items())
        [(11, [9, 7]), (22, [4, 2])]

    We'll change the message to have yet a newer value for recipients and
    create an event with the message as participant2.

        >>> message.recipients = [5, 6]
        >>> event = EventStub(None, message)

    When we call updatePersonsResponsibleIndex with this event, we see that
    that persons_responsible index entry has been updated for that message.

        >>> updatePersonsResponsibleIndex(event)
        >>> sorted(catalog['persons_responsible'].documents_to_values.items())
        [(11, [5, 6]), (22, [4, 2])]

    We'll do the same two tests for the goal.

        >>> goal.persons_responsible = [0, 1]
        >>> event = EventStub(goal, None)
        >>> updatePersonsResponsibleIndex(event)
        >>> sorted(catalog['persons_responsible'].documents_to_values.items())
        [(11, [5, 6]), (22, [0, 1])]

        >>> goal.persons_responsible = [99, 55]
        >>> event = EventStub(None, goal)
        >>> updatePersonsResponsibleIndex(event)
        >>> sorted(catalog['persons_responsible'].documents_to_values.items())
        [(11, [5, 6]), (22, [99, 55])]

    """


def setUp(test):
    setup.placefulSetUp()
    setup.setUpTraversal()
    app = AppStub()
    app.setSiteManager(LocalSiteManager(app))
    setSite(app)
    provideAdapter(lambda x: app, (None,), ISchoolToolApplication)
    provideAdapter(getAppCatalogs)
    provideAdapter(lambda student: ContactStub(student.first_name,
                                               student.last_name),
                   adapts=[IPerson],
                   provides=IContact)
    provideAdapter(lambda message: StudentStub('John', 'Deere'),
                   adapts=[interfaces.IInterventionMessage],
                   provides=IPerson)
    provideAdapter(lambda goal: StudentStub('Jane', 'Doe'),
                   adapts=[interfaces.IInterventionGoal],
                   provides=IPerson)
    provideAdapter(getMessagePersonsResponsible,
                   adapts=[interfaces.IInterventionMessage],
                   provides=interfaces.IInterventionPersonsResponsible)
    provideAdapter(getGoalPersonsResponsible,
                   adapts=[interfaces.IInterventionGoal],
                   provides=interfaces.IInterventionPersonsResponsible)
    provideAdapter(lambda x: SchoolYearStub(),
                   adapts=[interfaces.IInterventionMessage],
                   provides=interfaces.IInterventionSchoolYear)
    provideAdapter(lambda x: SchoolYearStub(),
                   adapts=[interfaces.IInterventionGoal],
                   provides=interfaces.IInterventionSchoolYear)
    provideAdapter(getMessageType,
                   adapts=[interfaces.IInterventionMessage],
                   provides=interfaces.IInterventionType)
    provideAdapter(getGoalType,
                   adapts=[interfaces.IInterventionGoal],
                   provides=interfaces.IInterventionType)


def tearDown(test):
    setup.placefulTearDown()


def test_suite():
    optionflags = (doctest.ELLIPSIS |
                   doctest.NORMALIZE_WHITESPACE |
                   doctest.REPORT_ONLY_FIRST_FAILURE)
    return doctest.DocTestSuite(setUp=setUp, tearDown=tearDown,
                                optionflags=optionflags)


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Functional tests for schooltool.intervention

"""

import unittest
import os
from datetime import date

from zope.interface import Interface, implements
from zope.publisher.browser import BrowserView
from zope.security.proxy import removeSecurityProxy

from schooltool.app.browser.ftests import setup
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.contact.interfaces import IContact
from schooltool.testing.analyze import queryHTML
from schooltool.testing.functional import collect_ftests
from schooltool.testing.functional import ZCMLLayer
from schooltool.basicperson.browser.ftests.setup import addPerson as addBasicPerson


dir = os.path.abspath(os.path.dirname(__file__))
filename = os.path.join(dir, '../ftesting.zcml')

intervention_functional_layer = ZCMLLayer(filename,
                                 __name__,
                                 'intervention_functional_layer')


def logInManager():
    return setup.logIn('manager', 'schooltool')


def addPerson(first_name, last_name, username, password, groups=None, browser=None):
    """Add a person.

    If username is not specified, it will be taken to be name.lower().

    If password is not specified, it will be taken to be username + 'pwd'.
    """
    if browser is None:
        browser = logInManager()
    addBasicPerson(first_name, last_name, username, password, groups, browser)

    fillInContactInfo(username, browser=browser,
                      email='%s@example.com' % username)
    browser.open('http://localhost/persons')


def fillInContactInfo(username, browser=None, email=None, language=None):
    if browser is None:
        browser = logInManager()
    browser.open('http://localhost/persons/%s' % username)
    # TODO: other needed fields
    contact_fields = {
        'form.widgets.email': email,
        'form.widgets.language': language,
        }
    browser.getLink('Contacts').click()
    browser.getLink('Edit').click()
    for name, value in contact_fields.items():
        if value is not None:
            browser.getControl(name=name).value = value
    browser.getControl('Apply').click()


def addCourseSectionMembers(course, section, teachers, students):
    setup.addCourse(course, '2005-2006')
    setup.addSection(course, '2005-2006', 'Spring', section)
    manager = logInManager()
    manager.getLink('Manage').click()
    manager.getLink('School Years').click()
    manager.getLink('2005-2006').click()
    manager.getLink('Spring').click()
    manager.getLink('Sections').click()
    manager.getLink(section).click()
    manager.getLink('edit instructors').click()
    for teacher in teachers:
        manager.getControl(teacher).click()
    manager.getControl('Add').click()
    manager.getControl('OK').click()
    manager.getLink('edit individuals').click()
    for student in students:
        manager.getControl(student).click()
    manager.getControl('Add').click()
    manager.getControl('OK').click()


def addAdvisors(student, advisors):
    manager = logInManager()
    for advisor in advisors:
        manager.getLink('Manage').click()
        manager.getLink('Persons').click()
        manager.getLink(student).click()
        manager.getLink('Advisors').click()
        manager.getControl(name='add_item.' + advisor).value = 'checked'
        manager.getControl('Add').click()


def removeAdvisors(student, advisors):
    manager = logInManager()
    for advisor in advisors:
        manager.getLink('Manage').click()
        manager.getLink('Persons').click()
        manager.getLink(student).click()
        manager.getLink('Advisors').click()
        manager.getControl(name='remove_item.' + advisor).value = 'checked'
        manager.getControl('Remove').click()


def _checkPersonsResponsible(browser, url, persons_responsible):
    browser.open(url)
    pre_checked_xpath = '//input[@type="checkbox"][@checked="checked"]/@name'
    for fld in queryHTML(pre_checked_xpath, browser.contents):
        browser.getControl(name=fld).value = False
    all_checkboxes_xpath = '//input[@type="checkbox"]/@name'
    for person in persons_responsible:
        for fld in queryHTML(all_checkboxes_xpath, browser.contents):
            if fld.startswith('persons.' + person):
                browser.getControl(name=fld).value = True


def addMessage(browser, url, persons_responsible, body='x'):
    _checkPersonsResponsible(browser, url, persons_responsible)
    browser.getControl(name='field.body').value = body
    browser.getControl(name='UPDATE_SUBMIT').click()


def addEditGoal(browser, url, persons_responsible, values=None):
    _checkPersonsResponsible(browser, url, persons_responsible)
    if values is None:
        values = ['x', 'x', 'x', 'x', 'x', date.today()]
    browser.getControl('Presenting concerns').value = values[0]
    browser.getControl(name='field.goal').value = values[1]
    browser.getControl('Strengths').value = values[2]
    browser.getControl('Indicators').value = values[3]
    browser.getControl('Intervention').value = values[4]
    browser.getControl('Timeline').value = str(values[5])
    browser.getControl(name='UPDATE_SUBMIT').click()


class TestOnlyUpdateResponsibleView(BrowserView):

    def __call__(self):
        persons = ISchoolToolApplication(None)['persons']
        if 'add' in self.request:
            username = self.request.get('add')
            contact = IContact(persons[username])
            self.responsible.add(contact)
        elif 'remove' in self.request:
            username = self.request.get('remove')
            contact = IContact(persons[username])
            self.responsible.remove(contact)


class TestOnlyUpdateRecipientsView(TestOnlyUpdateResponsibleView):

    def __init__(self, context, request):
        super(TestOnlyUpdateRecipientsView, self).__init__(context, request)
        self.responsible = removeSecurityProxy(self.context).recipients


class TestOnlyUpdatePersonsResponsibleView(TestOnlyUpdateResponsibleView):

    def __init__(self, context, request):
        super(TestOnlyUpdatePersonsResponsibleView, self).__init__(context, request)
        self.responsible = removeSecurityProxy(self.context).persons_responsible


class ActivitiesStub(object):
    def values(self):
        worksheet1 = WorksheetStub('Worksheet1', True)
        worksheet2 = WorksheetStub('Worksheet2', False)
        return [worksheet1, worksheet2]


class IWorksheetStub(Interface):
    pass


class WorksheetStub(object):
    implements(IWorksheetStub)

    def __init__(self, title, deployed):
        self.title = title
        self.deployed = deployed


class WorksheetAbsoluteURLStub(object):
    def __init__(self, worksheet, request):
        self.worksheet = worksheet

    def __call__(self):
        return self.worksheet.title


def getSectionIndependentActivities(section):
    if section.title == 'special title':
        return ActivitiesStub()


def test_suite():
    return collect_ftests(layer=intervention_functional_layer)


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from zope.catalog.interfaces import ICatalog
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from zope.security.proxy import removeSecurityProxy

from schooltool.app.catalog import AttributeCatalog
from schooltool.contact.interfaces import IContact
from schooltool.person.interfaces import IPerson
from schooltool.table.catalog import ConvertingIndex

import interfaces, intervention


class InterventionCatalog(AttributeCatalog):
    version = '1 - replaced catalog utility'
    interface = interfaces.IInterventionMarker
    attributes = ('created',)

    def setIndexes(self, catalog):
        super(InterventionCatalog, self).setIndexes(catalog)
        catalog['first_name'] = ConvertingIndex(converter=getStudentFirstName)
        catalog['last_name'] = ConvertingIndex(converter=getStudentLastName)
        catalog['persons_responsible'] = ConvertingIndex(
            converter=interfaces.IInterventionPersonsResponsible)
        catalog['schoolYearId'] = ConvertingIndex(converter=getSchoolYearId)
        catalog['intervention_type'] = ConvertingIndex(
            converter=interfaces.IInterventionType)


getInterventionCatalog = InterventionCatalog.get


def getStudentFirstName(obj):
    return IContact(IPerson(obj)).first_name


def getStudentLastName(obj):
    return IContact(IPerson(obj)).last_name


def getMessagePersonsResponsible(obj):
    util = getUtility(IIntIds)
    return [util.getId(contact) for contact in obj.recipients]


def getGoalPersonsResponsible(obj):
    # XXX not using removeSecurityProxy caused ZODB to crash
    # Cannot pickle <type 'zope.security._proxy._Proxy'> objects
    util = getUtility(IIntIds)
    return [util.getId(contact)
            for contact in removeSecurityProxy(obj.persons_responsible)]


def getMessageType(obj):
    return 'message'


def getGoalType(obj):
    return 'goal'


def getSchoolYearId(obj):
    sy = interfaces.IInterventionSchoolYear(obj)
    return sy.__name__


def messageGetter(obj):
    value = obj.body
    if len(value) > 50:
        value = value[:47] + '...'
    return value


def goalGetter(obj):
    value = obj.presenting_concerns
    if not value:
        value = obj.goal
    if len(value) > 50:
        value = value[:47] + '...'
    return value


def updatePersonsResponsibleIndex(event):
    if event.rel_type != intervention.URIResponsible:
        return
    if (interfaces.IInterventionMessage.providedBy(event.participant1) or
        interfaces.IInterventionGoal.providedBy(event.participant1)):
        obj = event.participant1
    else:
        obj = event.participant2

    util = getUtility(IIntIds)
    catalog = getInterventionCatalog()
    if catalog is None:
        return
    obj_intid = util.queryId(obj)
    if obj_intid is not None and obj_intid in catalog.extent:
        catalog.index_doc(obj_intid, obj)




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2008 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 2.

Fix bad dates created by message and goal constructors.
"""

import pytz

from zope.app.generations.utility import getRootFolder


def fixCreated(obj):
    if obj.created.tzinfo is None:
        obj.created = pytz.UTC.localize(obj.created)


def evolve(context):
    """Fixes message and goal creation dates to have tzinfo set."""
    app = getRootFolder(context)
    interventionRoot = app.get(u'schooltool.interventions', {})
    for schoolYear in interventionRoot.values():
        for student in schoolYear.values():
            for message in student['messages'].values():
                fixCreated(message)
            for goal in student['goals'].values():
                fixCreated(goal)




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2008 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 8.

Unregister old catalog utility.
"""

from zope.app.generations.utility import findObjectsProviding
from zope.app.publication.zopepublication import ZopePublication
from zope.component import queryUtility
from zope.component.hooks import getSite, setSite
from zope.traversing.api import traverse

from schooltool.app.interfaces import ISchoolToolApplication
from zope.catalog.interfaces import ICatalog


CATALOG_KEY = 'schooltool.intervention'


def evolve(context):
    root = context.connection.root().get(ZopePublication.root_name, None)

    old_site = getSite()
    apps = findObjectsProviding(root, ISchoolToolApplication)
    for app in apps:
        setSite(app)
        sm = app.getSiteManager()
        default = traverse(app, '++etc++site/default')

        util = queryUtility(ICatalog, name=CATALOG_KEY, default=None)
        if util is None:
            continue
        name = util.__name__
        sm.unregisterUtility(util, ICatalog, CATALOG_KEY)
        del default[name]

    setSite(old_site)




# coding=UTF8
#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2009 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Unit tests for schooltool.intervention.generations.evolve7
"""

from datetime import datetime
import unittest
import doctest

from persistent import Persistent
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.app.generations.utility import getRootFolder
from zope.app.testing import setup
from zope.component.hooks import getSite, setSite
from zope.container.btree import BTreeContainer
from zope.container.contained import Contained
from zope.interface import implements
from zope.site.folder import Folder

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.basicperson.person import BasicPerson
from schooltool.contact.contact import Contact
from schooltool.contact.interfaces import IContactable
from schooltool.generations.tests import ContextStub
from schooltool.generations.tests import setUp, tearDown
from schooltool.person.interfaces import IPerson

from schooltool.intervention.generations.tests import provideAdapters
from schooltool.intervention.generations.evolve7 import evolve
from schooltool.intervention.intervention import InterventionMessage
from schooltool.intervention.intervention import InterventionGoal


class ParentStub(Persistent, Contained):
    implements(IAttributeAnnotatable)
    first_name = 'Parent'
    last_name = 'Name'


def contactName(contact):
    return '%s %s' % (contact.first_name, contact.last_name)


def doctest_evolve7():
    """Evolution to generation 7.

    We need an app.

        >>> provideAdapters()
        >>> context = ContextStub(app)

    We need a teacher, a student and a parent.

        >>> persons = app[u'persons'] = BTreeContainer()
        >>> jteacher = persons['jteacher'] = BasicPerson('jteacher', 'John', 'Teacher')
        >>> jstudent = persons['jstudent'] = BasicPerson('jstudent', 'Jane', 'Student')
        >>> parent = persons['parent'] = ParentStub()
        >>> IContactable(jstudent).contacts.add(parent)

    Finally we need an intervention root, schoolyear and student.

        >>> root = app[u'schooltool.interventions'] = BTreeContainer()
        >>> year = root['thisyear'] = BTreeContainer()
        >>> student = year['jstudent'] = BTreeContainer()
        >>> messages = student['messages'] = BTreeContainer()
        >>> goals = student['goals'] = BTreeContainer()

    Our first test will be with a message and goal with all good ids.

        >>> message1 = messages['1'] = InterventionMessage('', '', '')
        >>> message1.__dict__['sender'] = 'jteacher'
        >>> message1.__dict__['recipients'] = ['jstudent', 'jstudent:1']

        >>> goal1 = goals['1'] = InterventionGoal('', '', '', '', '', '', '', 
        ...                                       '')
        >>> goal1.__dict__['creator'] = 'jteacher'
        >>> goal1.__dict__['_persons_responsible'] = ['jstudent', 'jstudent:1']
        >>> goal1.__dict__['at_one_time_responsible'] = ['jstudent', 'jstudent:1']

    We'll run the evolve script, testing the effected values before and after:

        >>> message1.__dict__['sender']
        'jteacher'
        >>> message1.__dict__['recipients']
        ['jstudent', 'jstudent:1']
        >>> goal1.__dict__['creator']
        'jteacher'
        >>> goal1.__dict__['_persons_responsible']
        ['jstudent', 'jstudent:1']
        >>> goal1.__dict__['at_one_time_responsible']
        ['jstudent', 'jstudent:1']

        >>> evolve(context)

        >>> 'sender' in message1.__dict__
        False
        >>> [contactName(contact) for contact in message1.sender]
        ['John Teacher']
        >>> 'recipients' in message1.__dict__
        False
        >>> [contactName(contact) for contact in message1.recipients]
        ['Jane Student', 'Parent Name']
        >>> 'creator' in goal1.__dict__
        False
        >>> [contactName(contact) for contact in goal1.creator]
        ['John Teacher']
        >>> '_persons_responsible' in goal1.__dict__
        False
        >>> [contactName(contact) for contact in goal1._persons_responsible]
        ['Jane Student', 'Parent Name']
        >>> 'at_one_time_responsible' in goal1.__dict__
        False
        >>> [contactName(contact) for contact in goal1.at_one_time_responsible]
        ['Jane Student', 'Parent Name']

    Our second test will be with a message and goal with invalid ids.

        >>> message2 = messages['2'] = InterventionMessage('', '', '')
        >>> message2.__dict__['sender'] = 'invalid_id'
        >>> message2.__dict__['recipients'] = ['invalid_id', 'jstudent:2']

        >>> goal2 = goals['2'] = InterventionGoal('', '', '', '', '', '', '', 
        ...                                       '')
        >>> goal2.__dict__['creator'] = 'invalid_id'
        >>> goal2.__dict__['_persons_responsible'] = ['invalid_id', 'jstudent:2']
        >>> goal2.__dict__['at_one_time_responsible'] = ['invalid_id', 'invalid_id:1']

    We'll run the evolve script, testing the effected values before and after:

        >>> message2.__dict__['sender']
        'invalid_id'
        >>> message2.__dict__['recipients']
        ['invalid_id', 'jstudent:2']
        >>> goal2.__dict__['creator']
        'invalid_id'
        >>> goal2.__dict__['_persons_responsible']
        ['invalid_id', 'jstudent:2']
        >>> goal2.__dict__['at_one_time_responsible']
        ['invalid_id', 'invalid_id:1']

        >>> evolve(context)

        >>> 'sender' in message2.__dict__
        False
        >>> [contactName(contact) for contact in message2.sender]
        []
        >>> 'recipients' in message2.__dict__
        False
        >>> [contactName(contact) for contact in message2.recipients]
        []
        >>> 'creator' in goal2.__dict__
        False
        >>> [contactName(contact) for contact in goal2.creator]
        []
        >>> '_persons_responsible' in goal2.__dict__
        False
        >>> [contactName(contact) for contact in goal2._persons_responsible]
        []
        >>> 'at_one_time_responsible' in goal2.__dict__
        False
        >>> [contactName(contact) for contact in goal2.at_one_time_responsible]
        []

    What if the intervention container doesn't exist yet in the
    application:

        >>> del app[u'schooltool.interventions']
        >>> context = ContextStub(app)
        >>> evolve(context)

    """


def test_suite():
    return unittest.TestSuite([
        doctest.DocTestSuite(setUp=setUp, tearDown=tearDown,
                             optionflags=doctest.ELLIPSIS
                             | doctest.NORMALIZE_WHITESPACE
                             | doctest.REPORT_NDIFF
                             | doctest.REPORT_ONLY_FIRST_FAILURE),
        ])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2009 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Evolve database to generation 4.

Fix goals to copy persons_responsible to newly created at_one_time_responsible
attribute as well as the internal attribute, _persons_responsible, that is
used to store the value for what is now a persons_responsible get/set property.
"""

from zope.app.generations.utility import findObjectsProviding
from zope.app.generations.utility import getRootFolder
from zope.component.hooks import getSite, setSite

from schooltool.app.interfaces import ISchoolToolApplication


def fixAtOneTimeResponsible(obj):
    dir(obj)
    obj._persons_responsible = obj.__dict__['persons_responsible']
    del obj.__dict__['persons_responsible']
    obj.at_one_time_responsible = obj._persons_responsible


def evolve(context):
    """Copy persons_responsible to newly created at_one_time_responsible."""
    root = getRootFolder(context)

    old_site = getSite()
    apps = findObjectsProviding(root, ISchoolToolApplication)
    for app in apps:
        setSite(app)
        interventionRoot = app.get(u'schooltool.interventions', {})
        for schoolYear in interventionRoot.values():
            for student in schoolYear.values():
                for goal in student['goals'].values():
                    fixAtOneTimeResponsible(goal)

    setSite(old_site)



# coding=UTF8
#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2009 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Unit tests for schooltool.intervention.generations.evolve8
"""

import unittest
import doctest

from zope.app.testing import setup
from zope.interface import implements
from zope.component import queryUtility
from zope.component.hooks import getSite, setSite
from zope.traversing.api import traverse
from zope.site import LocalSiteManager
from zope.site.folder import Folder
from zope.location import Location
from zope.catalog.interfaces import ICatalog

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.generations.tests import ContextStub
from schooltool.generations.tests import setUp, tearDown
from schooltool.intervention.generations.evolve8 import evolve, CATALOG_KEY


def registerLocalUtility(site, utility, name):
   manager = site.getSiteManager()
   default = traverse(site, '++etc++site/default')
   default['storage.key:'+name] = utility
   manager.registerUtility(utility, ICatalog, name)


class UtilityStub(Folder):
    def __init__(self, name):
        super(UtilityStub, self).__init__()
        self.name = name

    def __repr__(self):
        return '<%s (%s)>' % (self.__class__.__name__, self.name)


def doctest_evolve8():
    """Test evolution to generation 8.

    First, let's build ST app with local catalog utility.

        >>> context = ContextStub(app)

        >>> registerLocalUtility(app, UtilityStub(CATALOG_KEY), CATALOG_KEY)

        >>> print queryUtility(ICatalog, name=CATALOG_KEY, default=None)
        <UtilityStub (schooltool.intervention)>

    Set the site to something else and evolve.

        >>> context.root_folder['frob'] = frob = Folder()
        >>> frob.setSiteManager(LocalSiteManager(frob))
        >>> setSite(frob)

        >>> evolve(context)

    Active site was kept.

        >>> getSite() is frob
        True

    And catalogs in our app were unregistered.

        >>> setSite(app)
        >>> print queryUtility(ICatalog, name=CATALOG_KEY, default=None)
        None

    """


def test_suite():
    optionflags = (doctest.ELLIPSIS |
                   doctest.NORMALIZE_WHITESPACE |
                   doctest.REPORT_ONLY_FIRST_FAILURE)
    return doctest.DocTestSuite(setUp=setUp, tearDown=tearDown,
                                optionflags=optionflags)


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Student Intervention widgets.
"""
from zope.browser.interfaces import IAdding 
from zope.formlib.interfaces import MissingInputError
from zope.formlib.interfaces import IBrowserWidget, IInputWidget
from zope.formlib.interfaces import InputErrors
from zope.app.form.browser.submit import Update
from zope.browserpage import ViewPageTemplateFile
from zope.component import adapts, getUtility, queryUtility
from zope.i18n import translate
from zope.interface import implements
from zope.security.proxy import removeSecurityProxy

from z3c.form.converter import BaseDataConverter
from z3c.form.interfaces import IFieldWidget, IDataConverter
from z3c.form.widget import Widget, FieldWidget

from schooltool.contact.interfaces import IContactable, IContact, IUniqueFormKey
from schooltool.contact.browser.relationship import get_relationship_title
from schooltool.course.interfaces import ISectionContainer
from schooltool.email.interfaces import IEmailUtility
from schooltool.group.interfaces import IGroupContainer
from schooltool.person.interfaces import IPerson
from schooltool.schoolyear.interfaces import ISchoolYear
from schooltool.term.interfaces import IDateManager

from schooltool.intervention import InterventionGettext as _
from schooltool.intervention import interfaces, intervention


class WidgetBase(object):
    """Base Class for all Widget Classes.

    XXX: why copy from zope.formlib.widget?"""

    implements(IBrowserWidget, IInputWidget)

    template = None
    _prefix = 'field.'
    _error = ''

    # See zope.formlib.interfaces.IWidget
    name = None
    label = property(lambda self: self.context.title)
    hint = property(lambda self: self.context.description)
    visible = True

    # See zope.formlib.interfaces.IInputWidget
    required = property(lambda self: self.context.required)

    def __init__(self, field, request):
        self.context = field
        self.request = request

    def isAddView(self):
        return IAdding.providedBy(self.context.context)

    def setRenderedValue(self, value):
        """See zope.formlib.interfaces.IWidget"""
        pass

    def setPrefix(self, prefix):
        """See zope.formlib.interfaces.IWidget"""
        self._prefix = prefix

    def applyChanges(self, content):
        """See zope.formlib.interfaces.IInputWidget"""
        field = self.context
        new_value = self.getInputValue()
        old_value = field.query(content, self)
        if new_value == old_value:
            return False
        field.set(content, new_value)
        return True

    def hasInput(self):
        """See zope.formlib.interfaces.IInputWidget"""
        return True

    def hasValidInput(self):
        """See zope.formlib.interfaces.IInputWidget"""
        return True

    def getInputValue(self):
        """See zope.formlib.interfaces.IInputWidget"""
        return None

    def error(self):
        """See zope.formlib.interfaces.IBrowserWidget"""
        return self._error

    def __call__(self):
        """See zope.formlib.interfaces.IBrowserWidget"""
        return self.template()


class PersonListWidget(WidgetBase):
    """Person List Widget"""

    template = ViewPageTemplateFile('templates/person_list.pt')

    def __init__(self, field, request):
        super(PersonListWidget, self).__init__(field, request)

        # set student from field, view type
        if self.isAddView():
            self.student = IPerson(field.context.context)
        else:
            self.student = IPerson(field.context)
        self.student = removeSecurityProxy(self.student)

        # establish email capability and error message
        utility = getUtility(IEmailUtility)
        self.enabled = utility.enabled()
        self.message = ""
        if not self.enabled:
            self.message = _('Email server is not enabled')
        else:
            sender = IContact(self.request.principal._person)
            if not sender.email:
                self.message = _('User has no email address')
                self.enabled = False

        # build responsible, notified and changed lists of contacts
        term = queryUtility(IDateManager).current_term
        advisors = [IContact(removeSecurityProxy(advisor))
                    for advisor in self.student.advisors]
        groups = IGroupContainer(ISchoolYear(term))
        administrators = [IContact(member)
                          for member in groups['administrators'].members]
        responsible = advisors + [contact for contact in administrators
                                  if contact not in advisors]
        for section in ISectionContainer(term).values():
            if self.student in section.members:
                for teacher in section.instructors:
                    if IContact(teacher) not in responsible:
                        responsible.append(IContact(teacher))
        notified = [IContact(self.student)]
        notified += [contact for contact in IContactable(self.student).contacts
                     if contact not in responsible]
        if self.isAddView():
            changed = []
        else:
            changed = [contact
                       for contact in field.context.at_one_time_responsible
                       if contact not in responsible + notified]

        # set choices attribute with complete list of contact choices
        self.choices = responsible + notified + changed

        # build value list according to view type, request
        valueList = self.getRequestValue()
        if not self.isAddView() and not Update in self.request:
            valueList = getattr(field.context, field.getName())

        # build final view attributes: responsible, notified and changed
        self.responsible = []
        self.notified = []
        self.changed = []
        for choice in self.choices:
            checked = ''
            if choice in valueList:
                checked = 'checked'

            value = intervention.contactName(choice)
            if choice in advisors:
                value = '%s (%s)' % (value, _('advisor'))
                if self.isAddView() and not Update in self.request:
                    checked = 'checked'

            if choice in IContactable(self.student).contacts:
                title = get_relationship_title(self.student, choice)
                value = '%s (%s)' % (value, title.lower())

            person = {
                'name': IUniqueFormKey(choice),
                'checked': checked,
                'value': value,
                'hasEmail': bool(removeSecurityProxy(choice).email),
                }

            if choice in responsible:
                self.responsible.append(person)
            elif choice in notified:
                self.notified.append(person)
            else:
                self.changed.append(person)

    def canEmail(self, person):
        return self.enabled and person['hasEmail']

    def getRequestValue(self):
        contacts = []
        for contact in self.choices:
            key = IUniqueFormKey(contact)
            if key in self.request:
                contacts.append(contact)
        return contacts

    def setPrefix(self, prefix):
        """See zope.formlib.interfaces.IWidget"""
        self._prefix = prefix
        self.name = prefix + self.context.username

    def getInputValue(self):
        """See zope.formlib.interfaces.IInputWidget"""
        contacts = self.getRequestValue()
        if not contacts and self.enabled:
            self._error = MissingInputError.__doc__
            raise MissingInputError(self.name, self.label, None)
        return contacts

    def hasValidInput(self):
        """See zope.formlib.interfaces.IInputWidget"""
        try:
            self.getInputValue()
            return True
        except InputErrors:
            return False


class PersonListZ3CFormWidget(Widget, PersonListWidget):
    """z3c form widget for PersonList field."""

    template = ViewPageTemplateFile('templates/f_person_list.pt')

    def __getitem__(self, attr):
        return getattr(self, attr)

    def addClass(self, klass):
        pass

    def getCSSClass(self, *args, **kw):
        pass

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        self.student = IPerson(self.context)
        sender = IPerson(self.request.principal, None)
        if sender is not None:
            sender = IContact(sender)

        # establish email capability and error message
        utility = getUtility(IEmailUtility)
        self.enabled = utility.enabled()
        self.message = ""
        if not self.enabled:
            self.message = _('Email server is not enabled')
        else:
            if sender is None or not sender.email:
                self.message = _('User has no email address')
                self.enabled = False

        # build responsible, notified and changed lists of contacts
        term = queryUtility(IDateManager).current_term
        advisors = [IContact(removeSecurityProxy(advisor))
                    for advisor in self.student.advisors]
        groups = IGroupContainer(ISchoolYear(term))
        administrators = [IContact(member)
                          for member in groups['administrators'].members
                          if IContact(member) not in advisors]
        responsible = advisors + administrators
        teachers = []
        for section in ISectionContainer(term).values():
            if self.student in section.members:
                for teacher in section.instructors:
                    teacher = IContact(teacher)
                    if teacher not in responsible:
                        responsible.append(teacher)
                        teachers.append(teacher)
        notified = [IContact(self.student)]
        notified += [contact for contact in IContactable(self.student).contacts
                     if contact not in responsible]
        if self.isAddView():
            changed = []
        else:
            changed = [contact
                       for contact in self.context.at_one_time_responsible
                       if contact not in responsible + notified]

        # set choices attribute with complete list of contact choices
        self.choices = responsible + notified + changed

        # build value list according to view type, request
        valueList = self.getRequestValue()
        if not self.isAddView() and not 'SUBMIT' in self.request:
            valueList = getattr(self.context, self.field.getName())

        # build final view attributes: responsible, notified and changed
        self.responsible = []
        self.notified = []
        self.changed = []
        for choice in self.choices:
            title = ''
            checked = ''
            if choice in valueList:
                checked = 'checked'

            if choice in advisors:
                title = _('advisor')
                if self.isAddView() and not 'SUBMIT' in self.request:
                    checked = 'checked'
            elif choice in administrators:
                title = _('administrator')
            elif choice in teachers:
                title = _('teacher')
            elif choice is IContact(self.student):
                title = _('student')
            elif choice in IContactable(self.student).contacts:
                title = get_relationship_title(self.student, choice)
            elif choice is sender:
                title = _('self')
            if title:
                value = '%s (%s)' % (intervention.contactName(choice),
                                     translate(title, context=self.request).lower())
            else:
                value = intervention.contactName(choice)

            person = {
                'name': IUniqueFormKey(choice),
                'checked': checked,
                'value': value,
                'hasEmail': bool(removeSecurityProxy(choice).email),
                }

            if choice in responsible:
                self.responsible.append(person)
            elif choice in notified:
                self.notified.append(person)
            else:
                self.changed.append(person)

        # finally call the z3c base class
        super(PersonListZ3CFormWidget, self).update()

    def extract(self):
        return self.getRequestValue()

    def isAddView(self):
        return not interfaces.IInterventionMarker.providedBy(self.context)

    def render(self):
        self.update()
        return self.template()


def PersonListFieldWidget(field, request):
    """IFieldWidget factory for MyWidget."""
    widget = PersonListZ3CFormWidget(request)
    return FieldWidget(field, removeSecurityProxy(widget))


class PersonListFieldDataConverter(BaseDataConverter):
    """A PersonListField data converter."""
    adapts(interfaces.IPersonListField, IFieldWidget)
    implements(IDataConverter)

    def toFieldValue(self, value):
        """See interfaces.IDataConverter"""
        if value == []:
            return self.field.missing_value
        return value


class GoalMetWidget(WidgetBase):
    """Goal Met Widget"""

    template = ViewPageTemplateFile('templates/goal_met.pt')
    label = property(lambda self: '')

    def __init__(self, field, request):
        super(GoalMetWidget, self).__init__(field, request)
        
        if self.isAddView():
            value = False
        else:
            value = getattr(field.context, field.getName())

        if Update in self.request:
            value = self.getInputValue()

        self.goal_not_met = not value and 'checked' or None
        self.goal_met = value and 'checked' or None

    def getInputValue(self):
        """See zope.formlib.interfaces.IInputWidget"""
        return self.request['goal_met'] == 'Yes'



# Make a package.

def registerTestSetup():
    from schooltool.testing import registry

    def addInterventionRoot(app):
        from schooltool.intervention import intervention
        app['schooltool.interventions'] = intervention.InterventionRoot()
    registry.register('ApplicationContainers', addInterventionRoot)

registerTestSetup()
del registerTestSetup

from zope.i18nmessageid import MessageFactory
InterventionGettext = MessageFactory("schooltool.intervention")

import schooltool.common

schooltool.common.register_lauchpad_project(__package__, 'schooltool.intervention')




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Unit tests for the schooltool.intervention.adapters.
"""
import unittest
import doctest

from zope.app.testing.placelesssetup import setUp, tearDown
from zope.component import provideAdapter
from zope.interface import implements

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.intervention import adapters


def doctest_PersonRemovedSubsciber():
    """
    First we need some stubs.

        >>> class PersonStub:
        ...     def __init__(self, username):
        ...         self.username = username

        >>> class AppStub(dict):
        ...    implements(ISchoolToolApplication)

    We need to create an application, intervention root, intervention school
    year and a couple intervention students.

        >>> app = AppStub()
        >>> app['schooltool.interventions'] = root = {}
        >>> root['2009'] = year = {}
        >>> year['student1'] = 'Student One'
        >>> year['student2'] = 'Student Two'

    We need an adapter to get the app.

        >>> provideAdapter(lambda x: app, (None,), ISchoolToolApplication)

    We'll load the subscriber with student1 and see how the intervention
    student object will disappear.

        >>> student1 = PersonStub('student1')
        >>> subscriber = adapters.PersonRemovedSubsciber(None, student1)
        >>> subscriber()
        >>> sorted(year.keys())
        ['student2']
    """


def doctest_SchoolYearRemovedSubsciber():
    """
    First we need some stubs.

        >>> class SchoolYearStub:
        ...     def __init__(self, name):
        ...         self.__name__ = name

        >>> class AppStub(dict):
        ...    implements(ISchoolToolApplication)

    We need to create an application, an intervention root and a couple
    intervention schoolyears.

        >>> app = AppStub()
        >>> app['schooltool.interventions'] = root = {}
        >>> root['2008'] = {}
        >>> root['2009'] = {}

    We need an adapter to get the app.

        >>> provideAdapter(lambda x: app, (None,), ISchoolToolApplication)

    We'll load the subscriber with student1 and see how the intervention
    student object will disappear.

        >>> year = SchoolYearStub('2009')
        >>> subscriber = adapters.SchoolYearRemovedSubsciber(None, year)
        >>> subscriber()
        >>> sorted(root.keys())
        ['2008']
    """


def test_suite():
    optionflags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    suite = doctest.DocTestSuite(optionflags=optionflags,
                                 setUp=setUp, tearDown=tearDown)
    return unittest.TestSuite([suite])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




# coding=UTF8
#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2009 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Unit tests for schooltool.intervention.generations.evolve6
"""

import unittest
import doctest

from zope.app.testing import setup
from zope.component.hooks import setSite
from zope.container.btree import BTreeContainer
from zope.interface import implements
from zope.site.folder import Folder

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.generations.tests import ContextStub
from schooltool.generations.tests import setUp, tearDown
from schooltool.intervention.generations.evolve6 import evolve


class AppStub(Folder):
    implements(ISchoolToolApplication)


def doctest_evolve6():
    """Evolution to generation 6.

    First, we'll set up the app object:

        >>> context = ContextStub(app)

    We'll set up our test with data that will be effected by running the
    evolve script:

        >>> app[u'schooltool.schoolyear'] = BTreeContainer()
        >>> app[u'schooltool.schoolyear'][u'2009'] = BTreeContainer()
        >>> root = app[u'schooltool.interventions'] = BTreeContainer()
        >>> root['2008'] = BTreeContainer()
        >>> root['2009'] = BTreeContainer()

    Finally, we'll run the evolve script, testing the effected values before and
    after:

        >>> sorted(root.keys())
        [u'2008', u'2009']

        >>> evolve(context)

        >>> sorted(root.keys())
        [u'2009']

    What if the intervention container doesn't exist yet in the
    application:

        >>> del app[u'schooltool.interventions']
        >>> context = ContextStub(app)
        >>> evolve(context)

    """


def test_suite():
    return unittest.TestSuite([
        doctest.DocTestSuite(setUp=setUp, tearDown=tearDown,
                             optionflags=doctest.ELLIPSIS
                             | doctest.NORMALIZE_WHITESPACE
                             | doctest.REPORT_NDIFF
                             | doctest.REPORT_ONLY_FIRST_FAILURE),
        ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""
Tests for generation scripts.
"""

from zope.annotation.interfaces import IAnnotatable
from zope.app.dependable.interfaces import IDependable
from zope.app.dependable import Dependable
from zope.app.testing.setup import setUpAnnotations
from zope.component import provideAdapter

from schooltool.basicperson.interfaces import IBasicPerson
from schooltool.contact.basicperson import getBoundContact
from schooltool.contact.basicperson import getPersonOfBoundContact
from schooltool.contact.contact import Contactable
from schooltool.relationship.annotatable import getRelationshipLinks
from schooltool.relationship.interfaces import IRelationshipLinks


def provideAdapters():
    setUpAnnotations()
    provideAdapter(getRelationshipLinks, adapts=(IAnnotatable,),
                   provides=IRelationshipLinks)
    provideAdapter(getBoundContact, adapts=(IBasicPerson,))
    provideAdapter(Contactable, adapts=(IBasicPerson,))
    provideAdapter(getPersonOfBoundContact)
    provideAdapter(Dependable, adapts=(IAnnotatable,),
                   provides=IDependable)




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from zope.container.contained import NameChooser
from zope.container.interfaces import INameChooser
from zope.component import queryUtility, getMultiAdapter
from zope.component import adapter, adapts
from zope.interface import implementer, implements
from zope.lifecycleevent.interfaces import IObjectRemovedEvent
from zope.security.proxy import removeSecurityProxy
from zope.traversing.api import getParent

from schooltool.app.app import InitBase, StartUpBase
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.person.interfaces import IPerson
from schooltool.schoolyear.interfaces import ISchoolYear, ISchoolYearContainer
from schooltool.schoolyear.subscriber import ObjectEventAdapterSubscriber
from schooltool.term.interfaces import IDateManager

import interfaces, intervention


def setUpInterventions(app):
    app[u'schooltool.interventions'] = intervention.InterventionRoot()


class InterventionStartup(StartUpBase):
    def __call__(self):
        if u'schooltool.interventions' not in self.app:
            setUpInterventions(self.app)


class InterventionInit(InitBase):
    """Create the InterventionRoot object."""

    def __call__(self):
        setUpInterventions(self.app)


@adapter(ISchoolToolApplication)
@implementer(interfaces.IInterventionRoot)
def getInterventionRoot(app):
    return app[u'schooltool.interventions']


@adapter(ISchoolYear)
@implementer(interfaces.IInterventionSchoolYear)
def getSchoolYearInterventionSchoolYear(schoolyear):
    app = ISchoolToolApplication(None)
    interventionRoot = interfaces.IInterventionRoot(app)
    try:
        interventionSchoolYear = interventionRoot[schoolyear.__name__]
    except KeyError:
        interventionSchoolYear = intervention.InterventionSchoolYear()
        interventionRoot[schoolyear.__name__] = interventionSchoolYear
    return interventionSchoolYear


@adapter(ISchoolToolApplication)
@implementer(interfaces.IInterventionSchoolYear)
def getSchoolToolApplicationInterventionSchoolYear(app):
    term = queryUtility(IDateManager).current_term
    schoolyear = ISchoolYear(term)
    return interfaces.IInterventionSchoolYear(schoolyear)


@adapter(interfaces.IInterventionSchoolYear)
@implementer(ISchoolYear)
def getInterventionSchoolYearSchoolYear(schoolyear):
    app = ISchoolToolApplication(None)
    return ISchoolYearContainer(app)[schoolyear.__name__]


@adapter(interfaces.IInterventionMarker)
@implementer(interfaces.IInterventionStudent)
def getMarkerInterventionStudent(marker):
    return interfaces.IInterventionStudent(marker.__parent__.__parent__)


@adapter(IPerson, ISchoolYear)
@implementer(interfaces.IInterventionStudent)
def getStudentYearInterventionStudent(student, schoolyear):
    interventionSchoolYear = interfaces.IInterventionSchoolYear(schoolyear)
    try:
        interventionStudent = interventionSchoolYear[student.__name__]
    except KeyError:
        interventionStudent = intervention.InterventionStudent()
        interventionSchoolYear[student.__name__] = interventionStudent
        interventionStudent[u'messages'] = intervention.InterventionMessages()
        interventionStudent[u'goals'] = intervention.InterventionGoals()
    return interventionStudent


@adapter(IPerson)
@implementer(interfaces.IInterventionStudent)
def getStudentInterventionStudent(student):
    app = ISchoolToolApplication(None)
    interventionSchoolYear = interfaces.IInterventionSchoolYear(app)
    schoolyear = ISchoolYear(interventionSchoolYear)
    interventionStudent = getMultiAdapter((student, schoolyear), 
        interfaces.IInterventionStudent)
    return interventionStudent


@adapter(interfaces.IStudentSchoolYearProxy)
@implementer(interfaces.IInterventionStudent)
def getSchoolYearProxyInterventionStudent(proxy):
    student = proxy.__parent__.__parent__
    app = ISchoolToolApplication(None)
    schoolyear = ISchoolYearContainer(app)[proxy.__name__]
    interventionStudent = getMultiAdapter((student, schoolyear), 
        interfaces.IInterventionStudent)
    return interventionStudent


@adapter(interfaces.IInterventionStudent)
@implementer(IPerson)
def getInterventionStudentStudent(interventionStudent):
    persons = ISchoolToolApplication(None)['persons']
    return persons[interventionStudent.__name__]


@adapter(interfaces.IInterventionMessages)
@implementer(IPerson)
def getInterventionMessagesStudent(interventionMessages):
    return IPerson(interventionMessages.__parent__)


@adapter(interfaces.IInterventionMessage)
@implementer(IPerson)
def getInterventionMessageStudent(interventionMessage):
    return IPerson(interventionMessage.__parent__)


@adapter(interfaces.IInterventionGoals)
@implementer(IPerson)
def getInterventionGoalsStudent(interventionGoals):
    return IPerson(interventionGoals.__parent__)


@adapter(interfaces.IInterventionGoal)
@implementer(IPerson)
def getInterventionGoalStudent(interventionGoal):
    return IPerson(interventionGoal.__parent__)


@adapter(interfaces.ISectionMessagesProxy)
@implementer(IPerson)
def getSectionMessagesProxyStudent(proxy):
    return removeSecurityProxy(proxy).student


@adapter(interfaces.IStudentMessagesGoalsProxy)
@implementer(IPerson)
def getSectionMessagesGoalsProxyStudent(proxy):
    return removeSecurityProxy(proxy.__parent__.__parent__).student


def getInterventionSchoolYearFromObj(obj):
    while not interfaces.IInterventionSchoolYear.providedBy(obj):
        if interfaces.IStudentSchoolYearProxy.providedBy(obj):
            return obj.year
        obj = getParent(obj)
    return obj


class SequenceNumberNameChooser(NameChooser):
    """A name chooser that returns a sequence number."""

    implements(INameChooser)

    def chooseName(self, name, obj):
        """See INameChooser."""
        numbers = [int(v.__name__) for v in self.context.values()
                   if v.__name__.isdigit()]
        if numbers:
            n = str(max(numbers) + 1)
        else:
            n = '1'
        self.checkName(n, obj)
        return n


class PersonRemovedSubsciber(ObjectEventAdapterSubscriber):
    adapts(IObjectRemovedEvent, IPerson)

    def __call__(self):
        app = ISchoolToolApplication(None)
        interventionRoot = app[u'schooltool.interventions']
        for schoolYear in interventionRoot.values():
            if self.object.username in schoolYear:
                del schoolYear[self.object.username]


class SchoolYearRemovedSubsciber(ObjectEventAdapterSubscriber):
    adapts(IObjectRemovedEvent, ISchoolYear)

    def __call__(self):
        app = ISchoolToolApplication(None)
        interventionRoot = app[u'schooltool.interventions']
        if self.object.__name__ in interventionRoot:
            del interventionRoot[self.object.__name__]




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2005 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Unit tests for schooltool.intervention

"""

import unittest
import doctest

from zope.annotation.interfaces import IAnnotatable
from zope.app.testing import setup
from zope.component import provideAdapter, Interface
from zope.container.btree import BTreeContainer

from schooltool.app.app import getSchoolToolApplication
from schooltool.basicperson.interfaces import IBasicPerson
from schooltool.contact.basicperson import getBoundContact
from schooltool.contact.basicperson import getPersonOfBoundContact
from schooltool.contact.contact import Contactable
from schooltool.generations.tests import setUp as packageSetUp
from schooltool.generations.tests import tearDown
from schooltool.relationship.interfaces import IRelationshipLinks
from schooltool.relationship.annotatable import getRelationshipLinks
from schooltool.schoolyear.schoolyear import getSchoolYearContainer
from schooltool.schoolyear.schoolyear import SchoolYearInit
from schooltool.term.interfaces import ITermContainer
from schooltool.term.term import getTermContainer
from schooltool.term.term import getSchoolYearForTerm

from schooltool.intervention import interfaces, adapters

optionflags = (doctest.ELLIPSIS|doctest.REPORT_ONLY_FIRST_FAILURE|
               doctest.NORMALIZE_WHITESPACE)

def setUp(test):
    packageSetUp(test)
    provideAdapter(getSchoolToolApplication)
    provideAdapter(getTermContainer, [Interface], ITermContainer)
    provideAdapter(getSchoolYearForTerm)
    provideAdapter(getSchoolYearContainer)
    provideAdapter(getRelationshipLinks, (IAnnotatable,), IRelationshipLinks)
    provideAdapter(factory=Contactable, adapts=[IBasicPerson])
    provideAdapter(factory=getBoundContact, adapts=[None])
    provideAdapter(getPersonOfBoundContact)

    provideAdapter(adapters.getInterventionRoot)
    provideAdapter(adapters.getSchoolYearInterventionSchoolYear)
    provideAdapter(adapters.getSchoolToolApplicationInterventionSchoolYear)
    provideAdapter(adapters.getInterventionSchoolYearSchoolYear)
    provideAdapter(adapters.getMarkerInterventionStudent)
    provideAdapter(adapters.getStudentYearInterventionStudent)
    provideAdapter(adapters.getStudentInterventionStudent)
    provideAdapter(adapters.getInterventionStudentStudent)
    provideAdapter(adapters.getInterventionMessagesStudent)
    provideAdapter(adapters.getInterventionMessageStudent)
    provideAdapter(adapters.getInterventionGoalsStudent)
    provideAdapter(adapters.getInterventionGoalStudent)

    app = test.globs['app']
    app['persons'] = BTreeContainer()
    app['messages'] = BTreeContainer()
    app['goals'] = BTreeContainer()
    SchoolYearInit(app)()
    adapters.InterventionInit(app)()


def test_suite():
    return unittest.TestSuite([
         doctest.DocFileSuite('../README.txt', 
         setUp=setUp, tearDown=tearDown, 
         optionflags=optionflags),
         ])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2008 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Upgrade scholtool.intervention to generation 5.

This evolution script was registering and updating catalogs as utilities.
Now catalog creation and index updating is managed after evolution,
during app startup.
"""


def evolve(context):
    pass

